@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2013, the codeeditor development team
 #
 # Pyzo is distributed under the terms of the 2-Clause BSD License.
@@ -13,23 +12,22 @@ from ..qt import QtGui, QtCore
 
 Qt = QtCore.Qt
 
-import pyzo
-from ..misc import ustr, ce_option
+from ..misc import ce_option
 from ..parsers.tokens import (
     CommentToken,
     IdentifierToken,
+    NonIdentifierToken,
     StringToken,
     UnterminatedStringToken,
 )
-from ..parsers.python_parser import MultilineStringToken
-from ..parsers import BlockState
+from ..parsers.python_parser import MultilineStringToken, stringLiteralPrefixes
 
 
-class MoveLinesUpDown(object):
+class MoveLinesUpDown:
     def keyPressEvent(self, event):
-        if event.key() in (Qt.Key_Up, Qt.Key_Down) and (
-            Qt.ControlModifier & event.modifiers()
-            and Qt.ShiftModifier & event.modifiers()
+        if event.key() in (Qt.Key.Key_Up, Qt.Key.Key_Down) and (
+            Qt.KeyboardModifier.ControlModifier & event.modifiers()
+            and Qt.KeyboardModifier.ShiftModifier & event.modifiers()
         ):
             cursor = self.textCursor()
             cursor.beginEditBlock()
@@ -47,21 +45,24 @@ class MoveLinesUpDown(object):
         end = cursor.selectionEnd()
 
         # Get text of selected blocks
-        cursor.setPosition(start, cursor.MoveAnchor)
-        cursor.movePosition(cursor.StartOfBlock, cursor.MoveAnchor)
-        cursor.setPosition(end, cursor.KeepAnchor)
-        cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
+        cursor.setPosition(start, cursor.MoveMode.MoveAnchor)
+        cursor.movePosition(cursor.MoveOperation.StartOfBlock, cursor.MoveMode.MoveAnchor)
+        cursor.setPosition(end, cursor.MoveMode.KeepAnchor)
+        cursor.movePosition(cursor.MoveOperation.EndOfBlock, cursor.MoveMode.KeepAnchor)
         text1 = cursor.selectedText()
         cursor.removeSelectedText()
         pos1 = cursor.position()
 
         # Move up/down
-        other = [cursor.NextBlock, cursor.PreviousBlock][int(bool(key == Qt.Key_Up))]
-        cursor.movePosition(other, cursor.MoveAnchor)
+        other = [
+            cursor.MoveOperation.NextBlock,
+            cursor.MoveOperation.PreviousBlock,
+        ][key == Qt.Key.Key_Up]
+        cursor.movePosition(other, cursor.MoveMode.MoveAnchor)
 
         # Select text of other block
-        cursor.movePosition(cursor.StartOfBlock, cursor.MoveAnchor)
-        cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
+        cursor.movePosition(cursor.MoveOperation.StartOfBlock, cursor.MoveMode.MoveAnchor)
+        cursor.movePosition(cursor.MoveOperation.EndOfBlock, cursor.MoveMode.KeepAnchor)
         text2 = cursor.selectedText()
         cursor.removeSelectedText()
         pos2 = cursor.position()
@@ -71,10 +72,10 @@ class MoveLinesUpDown(object):
         pos3 = cursor.position()
 
         # Move back
-        if key == Qt.Key_Up:
-            cursor.movePosition(cursor.NextBlock, cursor.MoveAnchor)
+        if key == Qt.Key.Key_Up:
+            cursor.movePosition(cursor.MoveOperation.NextBlock, cursor.MoveMode.MoveAnchor)
         else:
-            cursor.setPosition(pos1, cursor.MoveAnchor)
+            cursor.setPosition(pos1, cursor.MoveMode.MoveAnchor)
             pos2 += len(text2)
             pos3 += len(text2)
 
@@ -82,20 +83,20 @@ class MoveLinesUpDown(object):
         cursor.insertText(text2)
 
         # Leave original lines selected for continued movement
-        cursor.setPosition(pos2, cursor.MoveAnchor)
-        cursor.setPosition(pos3, cursor.KeepAnchor)
+        cursor.setPosition(pos2, cursor.MoveMode.MoveAnchor)
+        cursor.setPosition(pos3, cursor.MoveMode.KeepAnchor)
         self.setTextCursor(cursor)
 
 
-class ScrollWithUpDownKeys(object):
+class ScrollWithUpDownKeys:
     def keyPressEvent(self, event):
         if (
-            event.key() in (Qt.Key_Up, Qt.Key_Down)
-            and Qt.ControlModifier == event.modifiers()
+            event.key() in (Qt.Key.Key_Up, Qt.Key.Key_Down)
+            and Qt.KeyboardModifier.ControlModifier == event.modifiers()
         ):
             s = self.verticalScrollBar()
             # h = self.cursorRect(self.textCursor()).height()
-            if event.key() == Qt.Key_Up:
+            if event.key() == Qt.Key.Key_Up:
                 s.setValue(s.value() + 1)
             else:
                 s.setValue(s.value() - 1)
@@ -104,56 +105,56 @@ class ScrollWithUpDownKeys(object):
             super().keyPressEvent(event)
 
 
-class HomeKey(object):
+class HomeKey:
     def keyPressEvent(self, event):
         # Home or shift + home
-        if event.key() == Qt.Key_Home and event.modifiers() in (
-            Qt.NoModifier,
-            Qt.ShiftModifier,
+        if event.key() == Qt.Key.Key_Home and event.modifiers() in (
+            Qt.KeyboardModifier.NoModifier,
+            Qt.KeyboardModifier.ShiftModifier,
         ):
             # Prepare
             cursor = self.textCursor()
-            shiftDown = event.modifiers() == Qt.ShiftModifier
-            moveMode = [cursor.MoveAnchor, cursor.KeepAnchor][shiftDown]
+            shiftDown = event.modifiers() == Qt.KeyboardModifier.ShiftModifier
+            moveMode = [cursor.MoveMode.MoveAnchor, cursor.MoveMode.KeepAnchor][shiftDown]
             # Get leading whitespace
-            text = ustr(cursor.block().text())
+            text = cursor.block().text()
             leadingWhitespace = text[: len(text) - len(text.lstrip())]
             # Get current position and move to start of whitespace
             i = cursor.positionInBlock()
-            cursor.movePosition(cursor.StartOfBlock, moveMode)
-            cursor.movePosition(cursor.Right, moveMode, len(leadingWhitespace))
+            cursor.movePosition(cursor.MoveOperation.StartOfBlock, moveMode)
+            cursor.movePosition(cursor.MoveOperation.Right, moveMode, len(leadingWhitespace))
             # If we were alread there, move to start of block
             if cursor.positionInBlock() == i:
-                cursor.movePosition(cursor.StartOfBlock, moveMode)
+                cursor.movePosition(cursor.MoveOperation.StartOfBlock, moveMode)
             # Done
             self.setTextCursor(cursor)
         else:
             super().keyPressEvent(event)
 
 
-class EndKey(object):
+class EndKey:
     def keyPressEvent(self, event):
-        if event.key() == Qt.Key_End and event.modifiers() in (
-            Qt.NoModifier,
-            Qt.ShiftModifier,
+        if event.key() == Qt.Key.Key_End and event.modifiers() in (
+            Qt.KeyboardModifier.NoModifier,
+            Qt.KeyboardModifier.ShiftModifier,
         ):
             # Prepare
             cursor = self.textCursor()
-            shiftDown = event.modifiers() == Qt.ShiftModifier
-            moveMode = [cursor.MoveAnchor, cursor.KeepAnchor][shiftDown]
+            shiftDown = event.modifiers() == Qt.KeyboardModifier.ShiftModifier
+            moveMode = [cursor.MoveMode.MoveAnchor, cursor.MoveMode.KeepAnchor][shiftDown]
             # Get current position and move to end of line
             i = cursor.positionInBlock()
-            cursor.movePosition(cursor.EndOfLine, moveMode)
+            cursor.movePosition(cursor.MoveOperation.EndOfLine, moveMode)
             # If alread at end of line, move to end of block
             if cursor.positionInBlock() == i:
-                cursor.movePosition(cursor.EndOfBlock, moveMode)
+                cursor.movePosition(cursor.MoveOperation.EndOfBlock, moveMode)
             # Done
             self.setTextCursor(cursor)
         else:
             super().keyPressEvent(event)
 
 
-class NumpadPeriodKey(object):
+class NumpadPeriodKey:
     """
     If the numpad decimal separator key is pressed, always insert
     a period (.) even if due to localization that key is mapped to a
@@ -166,13 +167,13 @@ class NumpadPeriodKey(object):
     def keyPressEvent(self, event):
         # Check for numpad comma
         if (
-            event.key() == QtCore.Qt.Key_Comma
-            and event.modifiers() & QtCore.Qt.KeypadModifier
+            event.key() == QtCore.Qt.Key.Key_Comma
+            and event.modifiers() & QtCore.Qt.KeyboardModifier.KeypadModifier
         ):
             # Create a new QKeyEvent to substitute the original one
             event = QtGui.QKeyEvent(
                 event.type(),
-                QtCore.Qt.Key_Period,
+                QtCore.Qt.Key.Key_Period,
                 event.modifiers(),
                 ".",
                 event.isAutoRepeat(),
@@ -182,7 +183,7 @@ class NumpadPeriodKey(object):
         super().keyPressEvent(event)
 
 
-class Indentation(object):
+class Indentation:
     def __cursorIsInLeadingWhitespace(self, cursor=None):
         """
         Checks wether the given cursor is in the leading whitespace of a block, i.e.
@@ -193,7 +194,7 @@ class Indentation(object):
             cursor = self.textCursor()
 
         # Get the text of the current block up to the cursor
-        textBeforeCursor = ustr(cursor.block().text())[: cursor.positionInBlock()]
+        textBeforeCursor = cursor.block().text()[: cursor.positionInBlock()]
         return (
             textBeforeCursor.lstrip() == ""
         )  # If we trim it and it is empty, it's all whitespace
@@ -202,8 +203,8 @@ class Indentation(object):
         key = event.key()
         modifiers = event.modifiers()
         # Tab key
-        if key == Qt.Key_Tab:
-            if modifiers == Qt.NoModifier:
+        if key == Qt.Key.Key_Tab:
+            if modifiers == Qt.KeyboardModifier.NoModifier:
                 if (
                     self.textCursor().hasSelection()
                 ):  # Tab pressed while some area was selected
@@ -230,8 +231,8 @@ class Indentation(object):
         # position of the line), and there is no selection
         # dedent that line and move cursor to end of whitespace
         if (
-            key == Qt.Key_Backspace
-            and modifiers == Qt.NoModifier
+            key == Qt.Key.Key_Backspace
+            and modifiers == Qt.KeyboardModifier.NoModifier
             and self.__cursorIsInLeadingWhitespace()
             and not self.textCursor().atBlockStart()
             and not self.textCursor().hasSelection()
@@ -245,13 +246,13 @@ class Indentation(object):
         # todo: Same for delete, I think not (what to do with the cursor?)
 
         # Auto-unindent
-        if key == Qt.Key_Delete:
+        if key == Qt.Key.Key_Delete:
             cursor = self.textCursor()
             if not cursor.hasSelection():
-                cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
+                cursor.movePosition(cursor.MoveOperation.EndOfBlock, cursor.MoveMode.KeepAnchor)
                 if not cursor.hasSelection() and cursor.block().next().isValid():
                     cursor.beginEditBlock()
-                    cursor.movePosition(cursor.NextBlock)
+                    cursor.movePosition(cursor.MoveOperation.NextBlock)
                     self.indentBlock(cursor, -99)  # dedent as much as we can
                     cursor.deletePreviousChar()
                     cursor.endEditBlock()
@@ -260,31 +261,23 @@ class Indentation(object):
         super().keyPressEvent(event)
 
 
-class AutoIndent(object):
+class AutoIndent:
     """
     Auto indentation. This extension only adds the autoIndent property, for the
     actual indentation, the editor should derive from some AutoIndenter object
     """
 
     def autoIndent(self):
-        """autoIndent()
-
-        Get whether auto indentation is enabled.
-
-        """
+        """Get whether auto indentation is enabled."""
         return self.__autoIndent
 
     @ce_option(True)
     def setAutoIndent(self, value):
-        """setAutoIndent(value)
-
-        Set whether to enable auto indentation.
-
-        """
+        """Set whether to enable auto indentation."""
         self.__autoIndent = bool(value)
 
 
-class PythonAutoIndent(object):
+class PythonAutoIndent:
     def keyPressEvent(self, event):
         super().keyPressEvent(event)
         if not self.autoIndent():
@@ -292,7 +285,7 @@ class PythonAutoIndent(object):
 
         # This extension code is run *after* key is processed by QPlainTextEdit
 
-        if event.key() in (Qt.Key_Enter, Qt.Key_Return):
+        if event.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
             cursor = self.textCursor()
 
             # Prevent in-block newlines (issue #482)
@@ -303,9 +296,9 @@ class PythonAutoIndent(object):
 
             previousBlock = cursor.block().previous()
             if previousBlock.isValid():
-                line = ustr(previousBlock.text())
+                line = previousBlock.text()
                 indent = line[: len(line) - len(line.lstrip())]
-                if line.endswith(":"):
+                if ":" in line:
                     # We only need to add indent if the : is not in a (multiline)
                     # string or comment. Therefore, find out what the syntax
                     # highlighter thinks of the previous line.
@@ -315,16 +308,16 @@ class PythonAutoIndent(object):
                     ppreviousState = (
                         ppreviousBlock.userState() if previousBlock.isValid() else 0
                     )
-                    lastElementToken = list(
-                        self.parser().parseLine(previousBlock.text(), ppreviousState)
-                    )[-1]
-                    # Because there's at least a : on that line, the list is never empty
-
-                    if not isinstance(
-                        lastElementToken,
-                        (CommentToken, UnterminatedStringToken, BlockState),
-                    ):
-                        # TODO: check correct identation (no mixed space/tabs)
+                    tokens = self.parser().parseLine(
+                        previousBlock.text(), ppreviousState
+                    )
+                    # because of the ":" on that line, there is at least one token
+                    t = tokens[-1]
+                    if isinstance(t, CommentToken):
+                        t = tokens[-2] if len(tokens) >= 2 else None
+                    if isinstance(t, NonIdentifierToken) and str(t).strip() == ":":
+                        # there is no need to check if the previous indent is the same
+                        # style as the added one, because we could not fix that anyways
                         if self.indentUsingSpaces():
                             indent += " " * self.indentWidth()
                         else:
@@ -334,7 +327,7 @@ class PythonAutoIndent(object):
                 self.setTextCursor(cursor)
 
 
-class SmartCopyAndPaste(object):
+class SmartCopyAndPaste:
     """
     Smart copy and paste allows copying and pasting blocks
 
@@ -343,7 +336,7 @@ class SmartCopyAndPaste(object):
     @staticmethod
     def __setCursorPositionAndAnchor(cursor, position, anchor):
         cursor.setPosition(anchor)
-        cursor.setPosition(position, cursor.KeepAnchor)
+        cursor.setPosition(position, cursor.MoveMode.KeepAnchor)
 
     @classmethod
     def __ensureCursorBeforeAnchor(cls, cursor):
@@ -365,8 +358,7 @@ class SmartCopyAndPaste(object):
         return anchorBeforeCursor
 
     def copy(self):
-        """
-        Smart copy: if selection is multi-line and in front of the start of the
+        """Smart copy: if selection is multi-line and in front of the start of the
         selection there is only whitespace, extend the selection to include only
         whitespace
         """
@@ -398,20 +390,26 @@ class SmartCopyAndPaste(object):
                 else:
                     self.__setCursorPositionAndAnchor(cursor, start, end)
 
+                # Setting the textcursor might scroll to another place
+                # (e.g. Ctrl+A, Ctrl+C would scroll to the very bottom).
+                # To prevent this, we store and then restore the position
+                # of the vertical scrollbar
+                sb = self.verticalScrollBar()
+                scrollbarPos = sb.value()
                 self.setTextCursor(cursor)
+                sb.setValue(scrollbarPos)
 
         # Call our supers copy slot to do the actual copying
         super().copy()
 
     def cut(self):
-        """
-        Cutting with smart-copy: the part that is copies is the same as self.copy(),
+        """Cutting with smart-copy: the part that is copied is the same as self.copy(),
         but the part that is removed is only the original selection
 
         see: Qt qtextcontrol.cpp, cut()
         """
         if (
-            self.textInteractionFlags() & QtCore.Qt.TextEditable
+            self.textInteractionFlags() & QtCore.Qt.TextInteractionFlag.TextEditable
         ) and self.textCursor().hasSelection():
             cursor = self.textCursor()
             self.copy()
@@ -420,8 +418,7 @@ class SmartCopyAndPaste(object):
             cursor.removeSelectedText()
 
     def paste(self):
-        """
-        Smart paste
+        """Smart paste
         If you paste on a position that has only whitespace in front of it,
         remove the whitespace before pasting. Combined with smart copy,
         this ensure indentation of the
@@ -429,8 +426,7 @@ class SmartCopyAndPaste(object):
         self._paste(keepSelection=False)
 
     def pasteAndSelect(self):
-        """
-        Smart paste
+        """Smart paste
         Like paste(), but keep the part that was pasted selected. This allows
         you to change the indentation after pasting using tab / shift-tab
         """
@@ -441,7 +437,7 @@ class SmartCopyAndPaste(object):
         cursor = self.textCursor()
 
         # Ensure that position is at start and anchor is at the end.
-        # This is required to ensure that with keepPositiobOnInsert
+        # This is required to ensure that with keepPositionOnInsert
         # set, the cursor will equal the pasted text after the pasting
         self.__ensureCursorBeforeAnchor(cursor)
 
@@ -469,7 +465,9 @@ class SmartCopyAndPaste(object):
 
                 # Move cursor2 to beginning of the line (selecting the whitespace)
                 # and remove the selection
-                cursor2.movePosition(cursor2.StartOfBlock, cursor2.KeepAnchor)
+                cursor2.movePosition(
+                    cursor2.MoveOperation.StartOfBlock, cursor2.MoveMode.KeepAnchor
+                )
                 cursor2.removeSelectedText()
 
         # Set the textcursor of this editor to the just-pasted selection
@@ -478,26 +476,46 @@ class SmartCopyAndPaste(object):
             self.setTextCursor(cursor)
 
 
-class AutoCloseQuotesAndBrackets(object):
+class AutoCloseQuotesAndBrackets:
     """
     Automatic insertion of quotes, parenthesis, braces and brackets
 
     """
 
+    def autoClose_Brackets(self):
+        return self.__autoClose_Brackets
+
+    @ce_option(False)
+    def setAutoClose_Brackets(self, value):
+        self.__autoClose_Brackets = value
+
+    def autoClose_Quotes(self):
+        return self.__autoClose_Quotes
+
+    @ce_option(False)
+    def setAutoClose_Quotes(self, value):
+        self.__autoClose_Quotes = value
+
     def _get_token_at_cursor(self, cursor=None, relpos=0):
         """Get token at the (current or given) cursor position. Can be None."""
         if cursor is None:
             cursor = self.textCursor()
         pos = cursor.positionInBlock() + relpos
-        tokens = cursor.block().userData().tokens
-        token = None
-        for token in tokens:
+        tokenAtCursor = None
+        for token in cursor.block().userData().tokens:
             if hasattr(token, "start"):
-                if token.start >= pos:
+                if token.start > pos:
                     break
-            elif getattr(token, "state", 0) in (1, 2):
-                token = MultilineStringToken()  # 1 and 2 are mls, by convention, sortof
-        return token
+                if token.start <= pos < token.end:
+                    tokenAtCursor = token
+                    break
+            if isinstance(token, MultilineStringToken):
+                # multiline string tokens can have a length
+                # of zero if the whole line is an empty comment
+                if token.start == token.end == 0:
+                    tokenAtCursor = token
+                    break
+        return tokenAtCursor
 
     def keyPressEvent(self, event):
         try:
@@ -518,8 +536,8 @@ class AutoCloseQuotesAndBrackets(object):
         char = event.text()
 
         #  brackets
-        if char in brackets and pyzo.config.settings.autoClose_Brackets:
-            # Dont autobracket inside comments and strings
+        if char in brackets and self.__autoClose_Brackets:
+            # Don't autobracket inside comments and strings
             if isinstance(
                 self._get_token_at_cursor(cursor),
                 (
@@ -569,10 +587,10 @@ class AutoCloseQuotesAndBrackets(object):
                     cursor.insertText(char)  # == super().keyPressEvent(event)
 
         # quotes
-        elif char in quotes and pyzo.config.settings.autoClose_Quotes:
+        elif char in quotes and self.__autoClose_Quotes:
             next_char = self.__getNextCharacter()
 
-            # Dont autoquote inside comments and multiline strings
+            # Don't autoquote inside comments and multiline strings
             # Only allow "doing our thing" when we're at the end of a normal string
             token = self._get_token_at_cursor(cursor)
             if isinstance(token, (CommentToken, MultilineStringToken)):
@@ -583,7 +601,7 @@ class AutoCloseQuotesAndBrackets(object):
                 return
 
             if cursor.selectedText() in ('"', "'"):
-                # Skip over char if its one char and a quote
+                # Skip over char if it's one char and a quote
                 self._moveCursorRight(1)
             elif cursor.selectedText():
                 # Surround selection with quotes, maybe even multi-line
@@ -599,22 +617,27 @@ class AutoCloseQuotesAndBrackets(object):
                 self._moveCursorRight(1)
             else:
                 # Only autoquote if we're next to whitespace, operator, quote
-                notok_token_types = (IdentifierToken,)
                 tokenL = self._get_token_at_cursor(cursor, -1)
                 tokenR = self._get_token_at_cursor(cursor, -0)
-                if isinstance(tokenL, notok_token_types) or isinstance(
-                    tokenR, notok_token_types
+
+                if (
+                    isinstance(tokenR, IdentifierToken)
+                    or isinstance(
+                        tokenL, StringToken
+                    )  # don't add a fourth " when typing " after two "
+                    or isinstance(tokenL, IdentifierToken)
+                    and str(tokenL) not in stringLiteralPrefixes
                 ):
                     super().keyPressEvent(event)
                     return
                 # Auto-close
                 cursor.insertText(char * 2)
                 self._moveCursorLeft(1)
-                # # Maybe handle tripple quotes (add 2 more if we now have 3 on the left)
+                # # Maybe handle triple quotes (add 2 more if we now have 3 on the left)
                 # Disabled: this feature too easily gets in the way
                 # if 'python' in self.parser().name().lower():
                 #     cursor = self.textCursor()
-                #     cursor.movePosition(cursor.PreviousCharacter, cursor.KeepAnchor, 3)
+                #     cursor.movePosition(cursor.MoveOperation.PreviousCharacter, cursor.MoveMode.KeepAnchor, 3)
                 #     if cursor.selectedText() == char * 3:
                 #         edit_cursor = self.textCursor()
                 #         edit_cursor.insertText(char * 2)
@@ -622,7 +645,7 @@ class AutoCloseQuotesAndBrackets(object):
 
         # remove whole couple of brackets when hitting backspace
         elif (
-            event.key() == Qt.Key_Backspace and pyzo.config.settings.autoClose_Brackets
+            event.key() == Qt.Key.Key_Backspace and self.__autoClose_Brackets
         ):
             if isinstance(
                 self._get_token_at_cursor(cursor),
@@ -654,30 +677,26 @@ class AutoCloseQuotesAndBrackets(object):
 
     def __getNextCharacter(self):
         cursor = self.textCursor()
-        cursor.movePosition(cursor.NoMove, cursor.MoveAnchor)  # rid selection
-        cursor.movePosition(cursor.NextCharacter, cursor.KeepAnchor)
+        cursor.movePosition(cursor.MoveOperation.NoMove, cursor.MoveMode.MoveAnchor)  # rid selection
+        cursor.movePosition(cursor.MoveOperation.NextCharacter, cursor.MoveMode.KeepAnchor)
         next_char = cursor.selectedText()
         return next_char
 
     def __getPreviousCharacter(self):
         cursor = self.textCursor()
-        cursor.movePosition(cursor.NoMove, cursor.MoveAnchor)  # rid selection
-        cursor.movePosition(cursor.PreviousCharacter, cursor.KeepAnchor)
+        cursor.movePosition(cursor.MoveOperation.NoMove, cursor.MoveMode.MoveAnchor)  # rid selection
+        cursor.movePosition(cursor.MoveOperation.PreviousCharacter, cursor.MoveMode.KeepAnchor)
         previous_char = cursor.selectedText()
         return previous_char
 
     def _moveCursorLeft(self, n):
-        """
-        Move cursor left between eg. brackets
-        """
+        """Move cursor left between eg. brackets"""
         cursor2 = self.textCursor()
-        cursor2.movePosition(cursor2.Left, cursor2.MoveAnchor, n)
+        cursor2.movePosition(cursor2.MoveOperation.Left, cursor2.MoveMode.MoveAnchor, n)
         self.setTextCursor(cursor2)
 
     def _moveCursorRight(self, n):
-        """
-        Move cursor out of eg. brackets
-        """
+        """Move cursor out of eg. brackets"""
         cursor2 = self.textCursor()
-        cursor2.movePosition(cursor2.Right, cursor2.MoveAnchor, n)
+        cursor2.movePosition(cursor2.MoveOperation.Right, cursor2.MoveMode.MoveAnchor, n)
         self.setTextCursor(cursor2)
