@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2013, the codeeditor development team
 #
 # Codeeditor is distributed under the terms of the 2-Clause BSD License.
@@ -32,23 +31,16 @@ import sys
 
 from . import tokens
 
-if sys.version_info[0] >= 3:
-    text_type = str
-else:
-    text_type = unicode  # noqa
 
+class BlockState:
+    """BlockState(state=0, info=None)
 
-class BlockState(object):
-    """ BlockState(state=0, info=None)
-    
     The blockstate object should be used by parsers to
     return the block state of the processed line.
-    
-    This would typically be the last item to be yielded, but this
-    it may also be yielded befor the last yielded token. One can even
-    yield multiple of these items, in which case the last one considered
-    valid.
-    
+
+    This would typically be the last token in a line, but this might also
+    be before the last token in a line. Even multiple BlockState objects
+    can be present in a line, in which case the last one is considered valid.
     """
 
     isToken = False
@@ -59,20 +51,18 @@ class BlockState(object):
 
     @property
     def state(self):
-        """ The integer value representing the block state.
-        """
+        """The integer value representing the block state."""
         return self._state
 
     @property
     def info(self):
-        """ Get the information corresponding to the block.
-        """
+        """Get the information corresponding to the block."""
         return self._info
 
 
 # Base parser class (needs to be defined before importing parser modules)
-class Parser(object):
-    """ Base parser class.
+class Parser:
+    """Base parser class.
     All parsers should inherit from this class.
     This base class generates a 'TextToken' for each line
     """
@@ -93,70 +83,45 @@ class Parser(object):
         return cls.getParserName()
 
     def parseLine(self, line, previousState=0):
-        """ parseLine(line, previousState=0)
-        
-        The method that should be implemented by the parser. The
+        """The method that should be implemented by the parser. The
         previousState argument can be used to determine how
         the previous block ended (e.g. for multiline comments). It
         is an integer, the meaning of which is only known to the
         specific parser.
-        
-        This method should yield token instances. The last token can
-        be a BlockState to specify the previousState for the
-        next block.
-        
+
+        This method should reaturn a list of token instances. The
+        last token can be a BlockState to specify the previousState
+        for the next block.
         """
 
-        yield tokens.TextToken(line, 0, len(line))
+        return [tokens.TextToken(line, 0, len(line))]
 
     def name(self):
-        """ name()
-        
-        Get the name of the parser.
-        
-        """
+        """Get the name of the parser."""
         name = self.__class__.__name__.lower()
         if name.endswith("parser"):
             name = name[:-6]
         return name
 
     def __repr__(self):
-        """ String representation of the parser.
-        """
-        return '<Parser for "%s">' % self.name()
+        """String representation of the parser."""
+        return '<Parser for "{}">'.format(self.name())
 
     def keywords(self):
-        """ keywords()
-        
-        Get a list of keywords valid for this parser.
-        
-        """
-        return [k for k in self._keywords]
+        """Get a list of keywords valid for this parser."""
+        return self._keywords[:]
 
     def filenameExtensions(self):
-        """ filenameExtensions()
-        
-        Get a list of filename extensions for which this parser
-        is appropriate.
-        
-        """
+        """Get a list of filename extensions for which this parser is appropriate."""
         return ["." + e.lstrip(".").lower() for e in self._extensions]
 
     def shebangKeywords(self):
-        """ shebangKeywords()
-        
-        Get a list of shebang keywords for which this parser
-        is appropriate.
-        
-        """
+        """Get a list of shebang keywords for which this parser is appropriate."""
         return self._shebangKeywords.copy()
 
     def getStyleElementDescriptions(cls):
-        """ getStyleElementDescriptions()
-        
-        This method returns a list of the StyleElementDescription
+        """This method returns a list of the StyleElementDescription
         instances used by this parser.
-        
         """
         descriptions = {}
         for token in cls.getUsedTokens(cls):
@@ -165,11 +130,7 @@ class Parser(object):
         return list(descriptions.values())
 
     def getUsedTokens(self):
-        """ getUsedTokens()
-        
-        Get a a list of token instances used by this parser.
-        
-        """
+        """Get a a list of token instances used by this parser."""
 
         # Get module object of the parser
         try:
@@ -189,12 +150,9 @@ class Parser(object):
         return [t() for t in tokenClasses]
 
     def _isTodoItem(self, text):
-        """ _isTodoItem(text)
-        
-        Get whether the given text (which should be a comment) represents
+        """Get whether the given text (which should be a comment) represents
         a todo item. Todo items start with "todo", "2do" or "fixme",
         optionally with a colon at the end.
-        
         """
         # Get first word
         word = text.lstrip().split(" ", 1)[0].rstrip(":")
@@ -207,7 +165,7 @@ class Parser(object):
 
 ## Import parsers statically
 # We could load the parser dynamically from the source files in the
-# directory, but this takes quite some effort to get righ when apps
+# directory, but this takes quite some effort to get right when apps
 # are frozen. This is doable (I do it in Visvis) but it requires the
 # user to specify the parser modules by hand when freezing an app.
 #
