@@ -1,20 +1,19 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2013, the codeeditor development team
 #
 # Pyzo is distributed under the terms of the 2-Clause BSD License.
 # The full license can be found in 'license.txt'.
 
-""" Modyule style
+""" Module style
 
-Provides basic functionaliy for styling.
+Provides basic functionality for styling.
 
 Styling is done using a dictionary of StyleFormat instances. Each
-such instance reprsents a certain element being styled (e.g. keywords,
+such instance represents a certain element being styled (e.g. keywords,
 line numbers, indentation guides).
 
 All possible style elements are represented using StyleElementDescription
 instances. These have a name, description and default format, which
-makes it easy to build a UI to allow the user to change the syle.
+makes it easy to build a UI to allow the user to change the style.
 
 """
 from .qt import QtGui, QtCore
@@ -23,13 +22,13 @@ Qt = QtCore.Qt
 
 
 class StyleElementDescription:
-    """ StyleElementDescription(name, defaultFormat, description)
-    
+    """StyleElementDescription(name, defaultFormat, description)
+
     Describes a style element by its name, default format, and description.
-    
+
     A style description is a simple placeholder for something
     that can be styled.
-    
+
     """
 
     def __init__(self, name, description, defaultFormat):
@@ -38,7 +37,7 @@ class StyleElementDescription:
         self._defaultFormat = StyleFormat(defaultFormat)
 
     def __repr__(self):
-        return '<"%s": "%s">' % (self.name, self.defaultFormat)
+        return '<"{}": "{}">'.format(self.name, self.defaultFormat)
 
     @property
     def name(self):
@@ -58,20 +57,20 @@ class StyleElementDescription:
 
 
 class StyleFormat:
-    """ StyleFormat(format='')
-    
+    """StyleFormat(format='')
+
     Represents the style format for a specific style element.
     A "style" is a dictionary that maps names (of style elements)
     to StyleFormat instances.
-    
+
     The given format can be a string or another StyleFormat instance.
     Style formats can be combined using their update() method.
-    
+
     A style format consists of multiple parts, where each "part" consists
     of a key and a value. The keys can be anything, depending
     on what kind of thing is being styled. The value can be obtained using
     the index operator (e.g. styleFomat['fore'])
-    
+
     For a few special keys, properties are defined that return the Qt object
     corresponding to the value. These values are also buffered to enable
     fast access. These keys are:
@@ -81,19 +80,19 @@ class StyleFormat:
       * italic: (bool) whether the text should be in italic
       * underline: (int) whether an underline should be used (and which one)
       * linestyle: (int) what line style to use (e.g. for indent guides)
-      * textCharFOrmat: (QTextCharFormat) for the syntax styles
-    
+      * textCharFormat: (QTextCharFormat) for the syntax styles
+
     The format neglects spaces and case. Parts are separated by commas
     or semicolons. If only a key is given it's value is interpreted
     as 'yes'. If only a color is given, its key is interpreted as 'fore'
     and back. Colors should be given using the '#' hex formatting.
-    
+
     An example format string: 'fore:#334, bold, underline:dotLine'
-    
+
     By calling str(styleFormatInstance) the string representing of the
     format can be obtained. By iterating over the instance, a series
     of key-value pairs is obtained.
-    
+
     """
 
     def __init__(self, format=""):
@@ -110,15 +109,12 @@ class StyleFormat:
         self._textCharFormat = None
 
     def __str__(self):
-        """ Get a (cleaned up) string representation of this style format.
-        """
-        parts = []
-        for key in self._parts:
-            parts.append("%s:%s" % (key, self._parts[key]))
+        """Get a (cleaned up) string representation of this style format."""
+        parts = ["{}:{}".format(key, self._parts[key]) for key in self._parts]
         return ", ".join(parts)
 
     def __repr__(self):
-        return '<StyleFormat "%s">' % str(self)
+        return '<StyleFormat "{}">'.format(self)
 
     def __getitem__(self, key):
         try:
@@ -127,19 +123,12 @@ class StyleFormat:
             raise KeyError("Invalid part key " + key + " for style format.")
 
     def __iter__(self):
-        """ Yields a series of tuples (key, val).
-        """
-        parts = []
-        for key in self._parts:
-            parts.append((key, self._parts[key]))
+        """Yields a series of tuples (key, val)."""
+        parts = [(key, self._parts[key]) for key in self._parts]
         return parts.__iter__()
 
     def update(self, format):
-        """ update(format)
-        
-        Update this style format with the given format.
-        
-        """
+        """Update this style format with the given format."""
 
         # Reset buffered values
         self._resetProperties()
@@ -148,11 +137,10 @@ class StyleFormat:
         if isinstance(format, StyleFormat):
             format = str(format)
 
-        # Split on ',' and ',', ignore spaces
-        styleParts = [p for p in format.replace("=", ":").replace(";", ",").split(",")]
+        # Split on ',' and ','
+        styleParts = format.replace("=", ":").replace(";", ",").split(",")
 
         for stylePart in styleParts:
-
             # Make sure it consists of identifier and value pair
             # e.g. fore:#xxx, bold:yes, underline:no
             if ":" not in stylePart:
@@ -194,33 +182,28 @@ class StyleFormat:
     @property
     def bold(self):
         if self._bold is None:
-            if self._getValueSafe("bold") in ["yes", "true"]:
-                self._bold = True
-            else:
-                self._bold = False
+            self._bold = self._getValueSafe("bold") in ["yes", "true"]
         return self._bold
 
     @property
     def italic(self):
         if self._italic is None:
-            if self._getValueSafe("italic") in ["yes", "true"]:
-                self._italic = True
-            else:
-                self._italic = False
+            self._italic = self._getValueSafe("italic") in ["yes", "true"]
         return self._italic
 
     @property
     def underline(self):
         if self._underline is None:
+            US = QtGui.QTextCharFormat.UnderlineStyle
             val = self._getValueSafe("underline")
             if val in ["yes", "true"]:
-                self._underline = QtGui.QTextCharFormat.SingleUnderline
+                self._underline = US.SingleUnderline
             elif val in ["dotted", "dots", "dotline"]:
-                self._underline = QtGui.QTextCharFormat.DotLine
+                self._underline = US.DotLine
             elif val in ["wave"]:
-                self._underline = QtGui.QTextCharFormat.WaveUnderline
+                self._underline = US.WaveUnderline
             else:
-                self._underline = QtGui.QTextCharFormat.NoUnderline
+                self._underline = US.NoUnderline
         return self._underline
 
     @property
@@ -228,13 +211,13 @@ class StyleFormat:
         if self._linestyle is None:
             val = self._getValueSafe("linestyle")
             if val in ["yes", "true"]:
-                self._linestyle = Qt.SolidLine
+                self._linestyle = Qt.PenStyle.SolidLine
             elif val in ["dotted", "dot", "dots", "dotline"]:
-                self._linestyle = Qt.DotLine
+                self._linestyle = Qt.PenStyle.DotLine
             elif val in ["dashed", "dash", "dashes", "dashline"]:
-                self._linestyle = Qt.DashLine
+                self._linestyle = Qt.PenStyle.DashLine
             else:
-                self._linestyle = Qt.SolidLine  # default to solid
+                self._linestyle = Qt.PenStyle.SolidLine  # default to solid
         return self._linestyle
 
     @property
@@ -248,7 +231,7 @@ class StyleFormat:
                 pass
             self._textCharFormat.setUnderlineStyle(self.underline)
             if self.bold:
-                self._textCharFormat.setFontWeight(QtGui.QFont.Bold)
+                self._textCharFormat.setFontWeight(QtGui.QFont.Weight.Bold)
             if self.italic:
                 self._textCharFormat.setFontItalic(True)
         return self._textCharFormat
