@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2016, the Pyzo development team
 #
 # Pyzo is distributed under the terms of the 2-Clause BSD License.
@@ -20,7 +19,6 @@ import pyzo
 from pyzo.core.icons import IconArtist
 from pyzo.core import commandline
 from pyzo.core.statusbar import StatusBar
-from pyzo import qt
 from pyzo.qt import QtCore, QtGui, QtWidgets
 from pyzo.core.splash import SplashWidget
 from pyzo.util import paths
@@ -30,7 +28,7 @@ from pyzo import translate
 
 class MainWindow(QtWidgets.QMainWindow):
     def __init__(self, parent=None, locale=None):
-        QtWidgets.QMainWindow.__init__(self, parent)
+        super().__init__(parent)
 
         self._closeflag = 0  # Used during closing/restarting
 
@@ -69,23 +67,44 @@ class MainWindow(QtWidgets.QMainWindow):
         pyzo.main = self
 
         # Init dockwidget settings
-        self.setTabPosition(QtCore.Qt.AllDockWidgetAreas, QtWidgets.QTabWidget.South)
+        self.setTabPosition(
+            QtCore.Qt.DockWidgetArea.AllDockWidgetAreas,
+            QtWidgets.QTabWidget.TabPosition.South,
+        )
         self.setDockOptions(
-            QtWidgets.QMainWindow.AllowTabbedDocks
-            | QtWidgets.QMainWindow.AllowNestedDocks
-            # |  QtWidgets.QMainWindow.AnimatedDocks
+            QtWidgets.QMainWindow.DockOption.AllowTabbedDocks
+            | QtWidgets.QMainWindow.DockOption.AllowNestedDocks
+            # |  QtWidgets.QMainWindow.DockOption.AnimatedDocks
         )
 
         # Set window atrributes
-        self.setAttribute(QtCore.Qt.WA_AlwaysShowToolTips, True)
+        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_AlwaysShowToolTips, True)
 
         # Load icons and fonts
         loadIcons()
         loadFonts()
 
-        # Set qt style and test success
+        # Set qt style, test success and detect Qt dark mode
         self.setQtStyle(None)  # None means init!
 
+        if pyzo.config.settings.theme == "":
+            # set light or dark mode when running Pyzo the first time
+            pyzo.config.settings.theme = "default_dark" if pyzo.darkQt else "default"
+
+        # detect dark syntax style (light text on dark background)
+        try:
+            theme = pyzo.themes[pyzo.config.settings.theme.lower()]["data"]
+            s = theme["editor.text"]  # e.g.: "fore:#657b83, back:#fff"
+            colors = dict(
+                [tuple(s3.strip() for s3 in s2.split(":")) for s2 in s.split(",")]
+            )
+            pyzo.darkSyntax = (
+                QtGui.QColor(colors["fore"]).lightness()
+                > QtGui.QColor(colors["back"]).lightness()
+            )
+        except KeyError:
+            pyzo.darkSyntax = False
+
         # Hold the splash screen if needed
         while time.time() < splash_timeout:
             QtWidgets.qApp.sendPostedEvents()
@@ -103,13 +122,14 @@ class MainWindow(QtWidgets.QMainWindow):
         self.restoreState()
         self.paintNow()
         self.restoreState()
+        pyzo.editors.restoreEditorState()
 
         # Present user with wizard if he/she is new.
         if False:  # pyzo.config.state.newUser:
             from pyzo.util.pyzowizard import PyzoWizard
 
             w = PyzoWizard(self)
-            w.show()  # Use show() instead of exec_() so the user can interact with pyzo
+            w.show()  # Use show() instead of exec() so the user can interact with pyzo
 
         # Create new shell config if there is None
         if not pyzo.config.shellConfigs2:
@@ -127,7 +147,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
     # To force drawing ourselves
     def paintEvent(self, event):
-        QtWidgets.QMainWindow.paintEvent(self, event)
+        super().paintEvent(event)
         self._ispainted = True
 
     def paintNow(self):
@@ -152,10 +172,6 @@ class MainWindow(QtWidgets.QMainWindow):
         # Instantiate tool manager
         pyzo.toolManager = ToolManager()
 
-        # Check to install conda now ...
-        # from pyzo.util.bootstrapconda import check_for_conda_env
-        # check_for_conda_env()
-
         # Instantiate and start source-code parser
         if pyzo.parser is None:
             pyzo.parser = codeparser.Parser()
@@ -167,13 +183,14 @@ class MainWindow(QtWidgets.QMainWindow):
 
         # Create floater for shell
         self._shellDock = dock = QtWidgets.QDockWidget(self)
+        DWF = dock.DockWidgetFeature
         if pyzo.config.settings.allowFloatingShell:
-            dock.setFeatures(dock.DockWidgetMovable | dock.DockWidgetFloatable)
+            dock.setFeatures(DWF.DockWidgetMovable | DWF.DockWidgetFloatable)
         else:
-            dock.setFeatures(dock.DockWidgetMovable)
+            dock.setFeatures(DWF.DockWidgetMovable)
         dock.setObjectName("shells")
         dock.setWindowTitle("Shells")
-        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, dock)
+        self.addDockWidget(QtCore.Qt.DockWidgetArea.RightDockWidgetArea, dock)
 
         # Create shell stack
         pyzo.shells = ShellStackWidget(self)
@@ -258,22 +275,22 @@ class MainWindow(QtWidgets.QMainWindow):
         pyzo.config.state.windowState = state
 
     def restoreGeometry(self, value=None):
-        # Restore window position and whether it is maximized
+        """Restore window position and whether it is maximized"""
 
         if value is not None:
             return super().restoreGeometry(value)
 
-        # No value give, try to get it from the config
+        # No value given, try to get it from the config
         if pyzo.config.state.windowGeometry:
             try:
                 geometry = pyzo.config.state.windowGeometry
                 geometry = base64.decodebytes(geometry.encode("ascii"))
                 self.restoreGeometry(geometry)
             except Exception as err:
-                print("Could not restore window geomerty: " + str(err))
+                print("Could not restore window geometry: " + str(err))
 
     def restoreState(self, value=None):
-        # Restore layout of dock widgets and toolbars
+        """Restore layout of dock widgets and toolbars"""
 
         if value is not None:
             return super().restoreState(value)
@@ -297,7 +314,7 @@ class MainWindow(QtWidgets.QMainWindow):
         if stylename is None:
             # Initialize
 
-            # Get native pallette (used below)
+            # Get native palette (used below)
             QtWidgets.qApp.nativePalette = QtWidgets.qApp.palette()
 
             # Obtain default style name
@@ -308,10 +325,8 @@ class MainWindow(QtWidgets.QMainWindow):
                 pass  # Use default style
             elif "macintosh" in pyzo.defaultQtStyleName.lower():
                 pass  # Use default style
-            elif qt.QT_VERSION > "5":
-                pyzo.defaultQtStyleName = "Fusion"
             else:
-                pyzo.defaultQtStyleName = "Cleanlooks"
+                pyzo.defaultQtStyleName = "Fusion"
 
             # Set style if there is no style yet
             if not pyzo.config.view.qtstyle:
@@ -333,6 +348,12 @@ class MainWindow(QtWidgets.QMainWindow):
         if qstyle:
             QtWidgets.qApp.setPalette(QtWidgets.qApp.nativePalette)
 
+        # detect dark mode for widgets (might be different than dark mode for syntax)
+        pal = QtWidgets.qApp.palette()
+        pyzo.darkQt = (
+            pal.window().color().lightness() < pal.windowText().color().lightness()
+        )
+
         # Done
         return qstyle
 
@@ -342,13 +363,6 @@ class MainWindow(QtWidgets.QMainWindow):
         # Are we restaring?
         restarting = time.time() - self._closeflag < 1.0  # noqa: F841
 
-        # Save settings
-        pyzo.saveConfig()
-        pyzo.command_history.save()
-
-        # Stop command server
-        commandline.stop_our_server()
-
         # Proceed with closing...
         result = pyzo.editors.closeAll()
         if not result:
@@ -359,6 +373,13 @@ class MainWindow(QtWidgets.QMainWindow):
             self._closeflag = True
             # event.accept()  # Had to comment on Windows+py3.3 to prevent error
 
+        # Save settings
+        pyzo.saveConfig()
+        pyzo.command_history.save()
+
+        # Stop command server
+        commandline.stop_our_server()
+
         # Proceed with closing shells
         pyzo.localKernelManager.terminateAll()
 
@@ -390,7 +411,7 @@ class MainWindow(QtWidgets.QMainWindow):
         #         print('Number of threads alive:', threading.activeCount())
 
         # Proceed as normal
-        QtWidgets.QMainWindow.closeEvent(self, event)
+        super().closeEvent(event)
 
     def restart(self):
         """Restart Pyzo."""
@@ -402,7 +423,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         if self._closeflag:
             # Get args
-            args = [arg for arg in sys.argv]
+            args = list(sys.argv)
 
             if not paths.is_frozen():
                 # Prepend the executable name (required on Linux)
@@ -428,33 +449,31 @@ class MainWindow(QtWidgets.QMainWindow):
             os.execv(sys.executable, args)
 
     def createPopupMenu(self):
-        # Init menu
-        menu = QtWidgets.QMenu()
+        # This menu pops up when right clicking on the menu bar
+        # or when right clicking on the title bar of a panel.
 
-        # Insert two items
-        for item in ["Editors", "Shells"]:
-            action = menu.addAction(item)
-            action.setCheckable(True)
-            action.setChecked(True)
-            action.setEnabled(False)
+        # We have to use the initialized menu from the parent class because creating
+        # a completely new QMenu would result in a memory error.
+        menu = super().createPopupMenu()
+        menu.clear()
 
-        # Insert tools
-        for tool in pyzo.toolManager.loadToolInfo():
-            action = menu.addAction(tool.name)
-            action.setCheckable(True)
-            action.setChecked(bool(tool.instance))
-            action.menuLauncher = tool.menuLauncher
+        # The build_callback function is necessary because without that, a simple lambda
+        # would bind the argument to the latest action of the for loop via the closure.
+        def build_callback(cb, action):
+            return lambda: cb(action.isChecked())
 
-        # Show menu and process result
-        a = menu.popup(QtGui.QCursor.pos())
-        if a:
-            a.menuLauncher(not a.menuLauncher(None))
+        # Add all tools, with checkmarks for those that are active
+        for tool in pyzo.toolManager.getToolInfo():
+            a = menu.addAction(tool.name)
+            a.setCheckable(True)
+            a.setChecked(bool(tool.instance))
+            a.triggered.connect(build_callback(tool.menuLauncher, a))
+
+        return menu
 
 
 def loadAppIcons():
-    """loadAppIcons()
-    Load the application iconsr.
-    """
+    """Load the application icons."""
     # Get directory containing the icons
     appiconDir = os.path.join(pyzo.pyzoDir, "resources", "appicons")
 
@@ -493,9 +512,7 @@ def loadAppIcons():
 
 
 def loadIcons():
-    """loadIcons()
-    Load all icons in the icon dir.
-    """
+    """Load all icons in the icon dir."""
     # Get directory containing the icons
     iconDir = os.path.join(pyzo.pyzoDir, "resources", "icons")
 
@@ -507,7 +524,7 @@ def loadIcons():
             try:
                 # Short and full name
                 name = fname.split(".")[0]
-                name = name.replace("pyzo_", "")  # discart prefix
+                name = name.replace("pyzo_", "")  # discard prefix
                 ffname = os.path.join(iconDir, fname)
                 # Create icon
                 icon = QtGui.QIcon()
@@ -516,13 +533,19 @@ def loadIcons():
                 pyzo.icons[name] = icon
             except Exception as err:
                 pyzo.icons[name] = dummyIcon
-                print("Could not load icon %s: %s" % (fname, str(err)))
+                print("Could not load icon {}: {}".format(fname, err))
+
+    artist = IconArtist("folder_page")
+    artist.addLayer("arrow_refresh")
+    pyzo.icons["reload_file_from_disk"] = artist.finish()
+
+    artist = IconArtist("page_white_copy")
+    artist.addLayer("overlay_disk")
+    pyzo.icons["save_copy_as"] = artist.finish()
 
 
 def loadFonts():
-    """loadFonts()
-    Load all fonts that come with Pyzo.
-    """
+    """Load all fonts that come with Pyzo."""
     import pyzo.codeeditor  # we need pyzo and codeeditor namespace here
 
     # Get directory containing the icons
@@ -543,14 +566,14 @@ def loadFonts():
                 try:
                     db.addApplicationFont(os.path.join(fontDir, fname))
                 except Exception as err:
-                    print("Could not load font %s: %s" % (fname, str(err)))
+                    print("Could not load font {}: {}".format(fname, err))
 
 
 class _CallbackEventHandler(QtCore.QObject):
     """Helper class to provide the callLater function."""
 
     def __init__(self):
-        QtCore.QObject.__init__(self)
+        super().__init__()
         self.queue = Queue()
 
     def customEvent(self, event):
@@ -566,13 +589,11 @@ class _CallbackEventHandler(QtCore.QObject):
 
     def postEventWithCallback(self, callback, *args):
         self.queue.put((callback, args))
-        QtWidgets.qApp.postEvent(self, QtCore.QEvent(QtCore.QEvent.User))
+        QtWidgets.qApp.postEvent(self, QtCore.QEvent(QtCore.QEvent.Type.User))
 
 
 def callLater(callback, *args):
-    """callLater(callback, *args)
-    Post a callback to be called in the main thread.
-    """
+    """Post a callback to be called in the main thread."""
     _callbackEventHandler.postEventWithCallback(callback, *args)
 
 
@@ -627,12 +648,12 @@ def screenshot(countdown=5):
 def _screenshot():
     # Grab
     print("SNAP!")
-    pix = QtGui.QPixmap.grabWindow(pyzo.main.winId())
-    # pix = QtGui.QPixmap.grabWidget(pyzo.main)
+    screen = QtWidgets.qApp.primaryScreen()
+    pix = screen.grabWindow(pyzo.main.winId())
     # Get name
     i = 1
     while i > 0:
-        name = "pyzo_screen_%s_%02i.png" % (sys.platform, i)
+        name = "pyzo_screen_{}_{:02d}.png".format(sys.platform, i)
         fname = os.path.join(os.path.expanduser("~"), name)
         if os.path.isfile(fname):
             i += 1
