@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2016, the Pyzo development team
 #
 # Pyzo is distributed under the terms of the 2-Clause BSD License.
@@ -37,9 +36,9 @@ from pyzo.core.kernelbroker import KernelInfo, Kernelmanager
 from pyzo.core.menu import ShellContextMenu
 
 
-# Interval for polling messages. Timer for each kernel. I found
-# that this one does not affect performance much
-POLL_TIMER_INTERVAL = 100  # 100ms 10Hz
+# Interval for polling messages. Timer for each kernel.
+POLL_TIMER_INTERVAL_IDLE = 100  # in ms; energy saving mode
+POLL_TIMER_INTERVAL_BUSY = 10  # in ms; faster updates
 
 # Maximum number of lines in the shell
 MAXBLOCKCOUNT = pyzo.config.advanced.shellMaxLines
@@ -52,12 +51,12 @@ class YotonEmbedder(QtCore.QObject):
     """Embed the Yoton event loop."""
 
     def __init__(self):
-        QtCore.QObject.__init__(self)
+        super().__init__()
         yoton.app.embed_event_loop(self.postYotonEvent)
 
     def postYotonEvent(self):
         try:
-            QtWidgets.qApp.postEvent(self, QtCore.QEvent(QtCore.QEvent.User))
+            QtWidgets.qApp.postEvent(self, QtCore.QEvent(QtCore.QEvent.Type.User))
         except Exception:
             pass  # If pyzo is shutting down, the app may be None
 
@@ -70,19 +69,16 @@ yotonEmbedder = YotonEmbedder()
 
 
 # Short constants for cursor movement
-A_KEEP = QtGui.QTextCursor.KeepAnchor
-A_MOVE = QtGui.QTextCursor.MoveAnchor
+A_KEEP = QtGui.QTextCursor.MoveMode.KeepAnchor
+A_MOVE = QtGui.QTextCursor.MoveMode.MoveAnchor
 
 # Instantiate a local kernel broker upon loading this module
 pyzo.localKernelManager = Kernelmanager(public=False)
 
 
 def finishKernelInfo(info, scriptFile=None):
-    """finishKernelInfo(info, scriptFile=None)
-
-    Get a copy of the kernel info struct, with the scriptFile
+    """Get a copy of the kernel info struct, with the scriptFile
     and the projectPath set.
-
     """
 
     # Make a copy, we do not want to change the original
@@ -97,13 +93,9 @@ def finishKernelInfo(info, scriptFile=None):
     # If the file browser is active, and has the check box
     #'add path to Python path' set, set the PROJECTPATH variable
     fileBrowser = pyzo.toolManager.getTool("pyzofilebrowser")
-    projectManager = pyzo.toolManager.getTool("pyzoprojectmanager")
     info.projectPath = ""
     if fileBrowser:
         info.projectPath = fileBrowser.getAddToPythonPath()
-    if projectManager and not info.projectPath:
-        # Only process project manager tool if file browser did not set a path.
-        info.projectPath = projectManager.getAddToPythonPath()
 
     return info
 
@@ -114,9 +106,6 @@ class ShellHighlighter(Highlighter):
     """
 
     def highlightBlock(self, line):
-        # Make sure this is a Unicode Python string
-        line = str(line)
-
         # Get previous state
         previousState = self.previousBlockState()
 
@@ -166,7 +155,7 @@ class ShellHighlighter(Highlighter):
             if specialinput:
                 pass  # Let the kernel decide formatting
             else:
-                tokens = list(parser.parseLine(line, previousState))
+                tokens = parser.parseLine(line, previousState)
                 bd.tokens = tokens
                 for token in tokens:
                     # Handle block state
@@ -180,14 +169,14 @@ class ShellHighlighter(Highlighter):
                             # print(repr(nameToFormat(token.name)))
                             continue
                         # Set format
-                        # format.setFontWeight(75)
+                        # format.setFontWeight(QtGui.QFont.Weight.Bold)
                         if token.start >= pos2:
                             self.setFormat(token.start, token.end - token.start, format)
 
             # Set prompt to bold
             if atCurrentPrompt:
                 format = QtGui.QTextCharFormat()
-                format.setFontWeight(75)
+                format.setFontWeight(QtGui.QFont.Weight.Bold)
                 self.setFormat(pos1, pos2 - pos1, format)
 
         # Get the indentation setting of the editors
@@ -198,8 +187,8 @@ class ShellHighlighter(Highlighter):
             # Mixed whitespace
             bd.indentation = 0
             format = QtGui.QTextCharFormat()
-            format.setUnderlineStyle(QtGui.QTextCharFormat.SpellCheckUnderline)
-            format.setUnderlineColor(QtCore.Qt.red)
+            format.setUnderlineStyle(QtGui.QTextCharFormat.UnderlineStyle.SpellCheckUnderline)
+            format.setUnderlineColor(QtCore.Qt.GlobalColor.red)
             format.setToolTip("Mixed tabs and spaces")
             self.setFormat(0, len(leadingWhitespace), format)
         elif ("\t" in leadingWhitespace and indentUsingSpaces) or (
@@ -208,8 +197,8 @@ class ShellHighlighter(Highlighter):
             # Whitespace differs from document setting
             bd.indentation = 0
             format = QtGui.QTextCharFormat()
-            format.setUnderlineStyle(QtGui.QTextCharFormat.SpellCheckUnderline)
-            format.setUnderlineColor(QtCore.Qt.blue)
+            format.setUnderlineStyle(QtGui.QTextCharFormat.UnderlineStyle.SpellCheckUnderline)
+            format.setUnderlineColor(QtCore.Qt.GlobalColor.blue)
             format.setToolTip("Whitespace differs from document setting")
             self.setFormat(0, len(leadingWhitespace), format)
         else:
@@ -272,7 +261,7 @@ class BaseShell(BaseTextCtrl):
 
         # Hard wrapping. QTextEdit allows hard wrapping at a specific column.
         # Unfortunately, QPlainTextEdit does not.
-        self.setWordWrapMode(QtGui.QTextOption.WrapAnywhere)
+        self.setWordWrapMode(QtGui.QTextOption.WrapMode.WrapAnywhere)
 
         # Limit number of lines
         self.setMaximumBlockCount(MAXBLOCKCOUNT)
@@ -281,7 +270,7 @@ class BaseShell(BaseTextCtrl):
         # is before the prompt
         self.cursorPositionChanged.connect(self.onCursorPositionChanged)
 
-        self.setFocusPolicy(Qt.TabFocus)  # See remark at mousePressEvent
+        self.setFocusPolicy(Qt.FocusPolicy.TabFocus)  # See remark at mousePressEvent
 
     ## Cursor stuff
 
@@ -296,14 +285,13 @@ class BaseShell(BaseTextCtrl):
             self.setReadOnly(False)
 
     def ensureCursorAtEditLine(self):
-        """
-        If the text cursor is before the beginning of the edit line,
+        """If the text cursor is before the beginning of the edit line,
         move it to the end of the edit line
         """
         cursor = self.textCursor()
         promptpos = self._cursor2.position()
         if cursor.position() < promptpos or cursor.anchor() < promptpos:
-            cursor.movePosition(cursor.End, A_MOVE)  # Move to end of document
+            cursor.movePosition(cursor.MoveOperation.End, A_MOVE)  # Move to end of document
             self.setTextCursor(cursor)
             self.onCursorPositionChanged()
 
@@ -324,7 +312,7 @@ class BaseShell(BaseTextCtrl):
             self.setFocus()
             return
 
-        if event.button() != QtCore.Qt.MidButton:
+        if event.button() != QtCore.Qt.MouseButton.MiddleButton:
             super().mousePressEvent(event)
 
     def contextMenuEvent(self, event):
@@ -332,8 +320,8 @@ class BaseShell(BaseTextCtrl):
         pass
 
     def mouseDoubleClickEvent(self, event):
-        BaseTextCtrl.mouseDoubleClickEvent(self, event)
-        self._handleClickOnFilename(event.pos())
+        super().mouseDoubleClickEvent(event)
+        self._handleClickOnFilename(event.position().toPoint())
 
     def _handleClickOnFilename(self, mousepos):
         """Check whether the text that is clicked is a filename
@@ -347,8 +335,8 @@ class BaseShell(BaseTextCtrl):
         pos = cursor.positionInBlock()
 
         # Get line of text for the cursor
-        cursor.movePosition(cursor.EndOfBlock, cursor.MoveAnchor)
-        cursor.movePosition(cursor.StartOfBlock, cursor.KeepAnchor)
+        cursor.movePosition(cursor.MoveOperation.EndOfBlock, cursor.MoveMode.MoveAnchor)
+        cursor.movePosition(cursor.MoveOperation.StartOfBlock, cursor.MoveMode.KeepAnchor)
         line = cursor.selectedText()
         if len(line) > 1024:
             return  # safety
@@ -367,7 +355,7 @@ class BaseShell(BaseTextCtrl):
                 return
 
             if sys.platform.startswith("win"):
-                if piece[1] != ":":
+                if piece[1] != ":" and piece[:2] != r"\\":
                     return
             else:
                 if not piece.startswith("/"):
@@ -396,8 +384,8 @@ class BaseShell(BaseTextCtrl):
         # IPython shows a few lines with the active line indicated by an arrow
         if linenr is None:
             for i in range(4):
-                cursor.movePosition(cursor.NextBlock, cursor.MoveAnchor)
-                cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
+                cursor.movePosition(cursor.MoveOperation.NextBlock, cursor.MoveMode.MoveAnchor)
+                cursor.movePosition(cursor.MoveOperation.EndOfBlock, cursor.MoveMode.KeepAnchor)
                 line = cursor.selectedText()
                 if len(line) > 1024:
                     continue  # safety
@@ -416,8 +404,8 @@ class BaseShell(BaseTextCtrl):
 
         # Select word here (in shell)
         cursor = ocursor
-        cursor.movePosition(cursor.Left, cursor.MoveAnchor, len(before))
-        cursor.movePosition(cursor.Right, cursor.KeepAnchor, len(piece))
+        cursor.movePosition(cursor.MoveOperation.Left, cursor.MoveMode.MoveAnchor, len(before))
+        cursor.movePosition(cursor.MoveOperation.Right, cursor.MoveMode.KeepAnchor, len(piece))
         self.setTextCursor(cursor)
 
         # For syntax errors we have the offset thingy in the file name
@@ -435,11 +423,11 @@ class BaseShell(BaseTextCtrl):
             editor = result._editor
             editor.gotoLine(linenr)
             cursor = editor.textCursor()
-            cursor.movePosition(cursor.StartOfBlock)
-            cursor.movePosition(cursor.EndOfBlock, cursor.KeepAnchor)
+            cursor.movePosition(cursor.MoveOperation.StartOfBlock)
+            cursor.movePosition(cursor.MoveOperation.EndOfBlock, cursor.MoveMode.KeepAnchor)
             editor.setTextCursor(cursor)
 
-    ##Indentation: override code editor behaviour
+    ## Indentation: override code editor behaviour
     def indentSelection(self):
         pass
 
@@ -448,7 +436,7 @@ class BaseShell(BaseTextCtrl):
 
     ## Key handlers
     def keyPressEvent(self, event):
-        if event.key() in [Qt.Key_Return, Qt.Key_Enter]:
+        if event.key() in [Qt.Key.Key_Return, Qt.Key.Key_Enter]:
             # First check if autocompletion triggered
             if self.potentiallyAutoComplete(event):
                 return
@@ -465,15 +453,15 @@ class BaseShell(BaseTextCtrl):
                 self.processLine()
                 return
 
-        if event.key() == Qt.Key_Escape:
+        if event.key() == Qt.Key.Key_Escape:
             # Escape clears command
             if not (self.autocompleteActive() or self.calltipActive()):
                 self.clearCommand()
 
-        if event.key() == Qt.Key_Home:
+        if event.key() == Qt.Key.Key_Home:
             # Home goes to the prompt.
             cursor = self.textCursor()
-            if event.modifiers() & Qt.ShiftModifier:
+            if event.modifiers() & Qt.KeyboardModifier.ShiftModifier:
                 cursor.setPosition(self._cursor2.position(), A_KEEP)
             else:
                 cursor.setPosition(self._cursor2.position(), A_MOVE)
@@ -482,19 +470,19 @@ class BaseShell(BaseTextCtrl):
             self.autocompleteCancel()
             return
 
-        if event.key() == Qt.Key_Insert:
+        if event.key() == Qt.Key.Key_Insert:
             # Don't toggle between insert mode and overwrite mode.
             return True
 
         # Ensure to not backspace / go left beyond the prompt
-        if event.key() in [Qt.Key_Backspace, Qt.Key_Left]:
+        if event.key() in [Qt.Key.Key_Backspace, Qt.Key.Key_Left]:
             self._historyNeedle = None
             if self.textCursor().position() == self._cursor2.position():
-                if event.key() == Qt.Key_Backspace:
+                if event.key() == Qt.Key.Key_Backspace:
                     self.textCursor().removeSelectedText()
                 return  # Ignore the key, don't go beyond the prompt
 
-        if event.key() in [Qt.Key_Up, Qt.Key_Down] and not self.autocompleteActive():
+        if event.key() in [Qt.Key.Key_Up, Qt.Key.Key_Down] and not self.autocompleteActive():
             # needle
             if self._historyNeedle is None:
                 # get partly-written-command
@@ -502,13 +490,13 @@ class BaseShell(BaseTextCtrl):
                 # Select text
                 cursor = self.textCursor()
                 cursor.setPosition(self._cursor2.position(), A_MOVE)
-                cursor.movePosition(cursor.End, A_KEEP)
+                cursor.movePosition(cursor.MoveOperation.End, A_KEEP)
                 # Update needle text
                 self._historyNeedle = cursor.selectedText()
                 self._historyStep = 0
 
             # Browse through history
-            if event.key() == Qt.Key_Up:
+            if event.key() == Qt.Key.Key_Up:
                 self._historyStep += 1
             else:  # Key_Down
                 self._historyStep -= 1
@@ -527,7 +515,7 @@ class BaseShell(BaseTextCtrl):
             # Replace text
             cursor = self.textCursor()
             cursor.setPosition(self._cursor2.position(), A_MOVE)
-            cursor.movePosition(cursor.End, A_KEEP)
+            cursor.movePosition(cursor.MoveOperation.End, A_KEEP)
             cursor.insertText(c)
 
             self.ensureCursorAtEditLine()
@@ -542,42 +530,41 @@ class BaseShell(BaseTextCtrl):
             self.ensureCursorAtEditLine()
 
         # Default behaviour: BaseTextCtrl
-        BaseTextCtrl.keyPressEvent(self, event)
+        super().keyPressEvent(event)
 
     ## Cut / Copy / Paste / Drag & Drop
 
     def cut(self):
         """Reimplement cut to only copy if part of the selected text
-        is not at the prompt."""
+        is not at the prompt.
+        """
 
         if self.isReadOnly():
             return self.copy()
         else:
-            return BaseTextCtrl.cut(self)
+            return super().cut()
 
     # def copy(self): # no overload needed
 
     def paste(self):
         """Reimplement paste to paste at the end of the edit line when
-        the position is at the prompt."""
+        the position is at the prompt.
+        """
         self.ensureCursorAtEditLine()
         # Paste normally
-        return BaseTextCtrl.paste(self)
+        return super().paste()
 
     def dragEnterEvent(self, event):
-        """
-        We only support copying of the text
-        """
+        """We only support copying of the text"""
         if event.mimeData().hasText():
-            event.setDropAction(QtCore.Qt.CopyAction)
+            event.setDropAction(QtCore.Qt.DropAction.CopyAction)
             event.accept()
 
     def dragMoveEvent(self, event):
         self.dragEnterEvent(event)
 
     def dropEvent(self, event):
-        """
-        The shell supports only a single line but the text may contain multiple
+        """The shell supports only a single line but the text may contain multiple
         lines. We insert at the editLine only the first non-empty line of the text
         """
         if event.mimeData().hasText():
@@ -590,7 +577,7 @@ class BaseShell(BaseTextCtrl):
 
             # Move the cursor to the position indicated by the drop location, but
             # ensure it is at the edit line
-            self.setTextCursor(self.cursorForPosition(event.pos()))
+            self.setTextCursor(self.cursorForPosition(event.position().toPoint()))
             self.ensureCursorAtEditLine()
 
             # Now insert the text
@@ -604,7 +591,7 @@ class BaseShell(BaseTextCtrl):
         """Clear all the previous output from the screen."""
         # Select from beginning of prompt to start of document
         self._cursor1.clearSelection()
-        self._cursor1.movePosition(self._cursor1.Start, A_KEEP)  # Keep anchor
+        self._cursor1.movePosition(self._cursor1.MoveOperation.Start, A_KEEP)  # Keep anchor
         self._cursor1.removeSelectedText()
         # Wrap up
         self.ensureCursorAtEditLine()
@@ -621,7 +608,7 @@ class BaseShell(BaseTextCtrl):
         # Select from prompt end to length and delete selected text.
         cursor = self.textCursor()
         cursor.setPosition(self._cursor2.position(), A_MOVE)
-        cursor.movePosition(cursor.End, A_KEEP)
+        cursor.movePosition(cursor.MoveOperation.End, A_KEEP)
         cursor.removeSelectedText()
         # Wrap up
         self.ensureCursorAtEditLine()
@@ -643,13 +630,11 @@ class BaseShell(BaseTextCtrl):
         return n, text2
 
     def _handleBackspacesOnList(self, texts):
-        """_handleBackspacesOnList(texts)
+        """Handle backspaces on a list of messages.
 
-        Handle backspaces on a list of messages. When printing
-        progress, many messages will simply replace each-other, which
-        means we can process them much more effectively than when they're
+        When printing progress, many messages will simply replace each-other,
+        which means we can process them much more effectively than when they're
         combined in a list.
-
         """
         # Init number of backspaces at the start
         N = 0
@@ -675,7 +660,7 @@ class BaseShell(BaseTextCtrl):
         if N:
             texts[0] = "\b" * N + texts[0]
 
-        # Return with empy elements popped
+        # Return with empty elements popped
         return [t for t in texts if t]
 
     def _handleBackspaces(self, text):
@@ -692,7 +677,7 @@ class BaseShell(BaseTextCtrl):
             if nb:
                 # Select what we remove and delete that
                 self._cursor1.clearSelection()
-                self._cursor1.movePosition(self._cursor1.Left, A_KEEP, nb)
+                self._cursor1.movePosition(self._cursor1.MoveOperation.Left, A_KEEP, nb)
                 self._cursor1.removeSelectedText()
 
         # Return result
@@ -741,7 +726,7 @@ class BaseShell(BaseTextCtrl):
             and not text[1:].startswith("\n")
             and not self._lastline_had_lf
         ):
-            cursor.movePosition(cursor.StartOfLine, cursor.KeepAnchor, 1)
+            cursor.movePosition(cursor.MoveOperation.StartOfLine, cursor.MoveMode.KeepAnchor, 1)
             cursor.removeSelectedText()
         # Is this new line ending in CR?
         self._lastline_had_cr = text.endswith("\r")
@@ -764,16 +749,13 @@ class BaseShell(BaseTextCtrl):
                 yield line
 
     def write(self, text, prompt=0, color=None):
-        """write(text, prompt=0, color=None)
-
-        Write to the shell. Fauto-ind
+        """Write to the shell.
 
         If prompt is 0 (default) the text is printed before the prompt. If
         prompt is 1, the text is printed after the prompt, the new prompt
         becomes null. If prompt is 2, the given text becomes the new prompt.
 
         The color of the text can also be specified (as a hex-string).
-
         """
 
         # From The Qt docs: Note that a cursor always moves when text is
@@ -781,7 +763,7 @@ class BaseShell(BaseTextCtrl):
         # keeps its position when text is inserted after the current position
         # of the cursor.
 
-        # Make sure there's text and make sure its a string
+        # Make sure there's text and make sure it's a string
         if not text:
             return
         if isinstance(text, bytes):
@@ -818,8 +800,9 @@ class BaseShell(BaseTextCtrl):
             self._cursor1.removeSelectedText()
             self._cursor2.setPosition(self._cursor1.position(), A_MOVE)
         elif prompt == 2:
-            # Insert text after prompt, inserted text becomes new prompt
-            self._cursor1.setPosition(self._cursor2.position(), A_MOVE)
+            # text becomes new prompt
+            self._cursor1.setPosition(self._cursor2.position(), A_KEEP)
+            self._cursor1.removeSelectedText()
             self._cursor1.setKeepPositionOnInsert(True)
             self._cursor2.setKeepPositionOnInsert(False)
             self._insertText(self._cursor1, text, format)
@@ -872,8 +855,8 @@ class BaseShell(BaseTextCtrl):
             cursor.insertText(text[i0:i1], format)
             i0 = i2
 
-            # The formay that we are now going to parse should apply to
-            # the text that follow it ...
+            # The format that we are now going to parse should be applied to
+            # the text that follows it ...
 
             # Get parameters
             try:
@@ -888,16 +871,16 @@ class BaseShell(BaseTextCtrl):
                 if param == 0:
                     format = QtGui.QTextCharFormat()
                 elif param == 1:
-                    format.setFontWeight(75)  # Bold
+                    format.setFontWeight(QtGui.QFont.Weight.Bold)
                 elif param == 2:
-                    format.setFontWeight(25)  # Faint
+                    format.setFontWeight(QtGui.QFont.Weight.Light)
                 elif param == 3:
                     format.setFontItalic(True)  # Italic
                 elif param == 4:
                     format.setFontUnderline(True)  # Underline
                 #
                 elif param == 22:
-                    format.setFontWeight(50)  # Not bold or faint
+                    format.setFontWeight(QtGui.QFont.Weight.Normal)  # Not bold or light
                 elif param == 23:
                     format.setFontItalic(False)  # Not italic
                 elif param == 24:
@@ -906,6 +889,8 @@ class BaseShell(BaseTextCtrl):
                 elif 30 <= param <= 37:  # Set foreground color
                     clr = CLRS[param - 30]
                     format.setForeground(QtGui.QColor(clr))
+                elif param == 39:  # Reset the foreground color
+                    format.clearForeground()
                 elif 40 <= param <= 47:
                     pass  # Cannot set background text in QPlainTextEdit
                 #
@@ -919,17 +904,18 @@ class BaseShell(BaseTextCtrl):
             # Process very long text more efficiently.
             # Insert per line (very long lines are split in smaller ones)
             if len(text) > 1024:
+                cursor.beginEditBlock()
                 for line in self._splitLinesForPrinting(text):
                     cursor.insertText(line, format)
+                cursor.endEditBlock()
             else:
                 cursor.insertText(text, format)
 
     ## Executing stuff
 
     def processLine(self, line=None, execute=True):
-        """processLine(self, line=None, execute=True)
+        """Process the given line or the current line at the prompt if not given.
 
-        Process the given line or the current line at the prompt if not given.
         Called when the user presses enter.
 
         If execute is False will not execute the command. This way
@@ -948,7 +934,7 @@ class BaseShell(BaseTextCtrl):
             # Select command
             cursor = self.textCursor()
             cursor.setPosition(self._cursor2.position(), A_MOVE)
-            cursor.movePosition(cursor.End, A_KEEP)
+            cursor.movePosition(cursor.MoveOperation.End, A_KEEP)
 
             # Sample the text from the prompt and remove it
             command = cursor.selectedText().replace("\u2029", "\n").rstrip("\n")
@@ -993,7 +979,7 @@ class PythonShell(BaseShell):
     debugStateChanged = QtCore.Signal(BaseShell)
 
     def __init__(self, parent, info):
-        BaseShell.__init__(self, parent)
+        super().__init__(parent)
 
         # Get standard info if not given.
         if info is None and pyzo.config.shellConfigs2:
@@ -1020,14 +1006,14 @@ class PythonShell(BaseShell):
         # messages are not so much bach-processed anymore. We should decide
         # on either method.
         self._timer = QtCore.QTimer(self)
-        self._timer.setInterval(POLL_TIMER_INTERVAL)  # ms
+        self._timer.setInterval(POLL_TIMER_INTERVAL_IDLE)  # ms
         self._timer.setSingleShot(False)
         self._timer.timeout.connect(self.poll)
         self._timer.start()
 
         # Add context menu
         self._menu = ShellContextMenu(shell=self, parent=self)
-        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.CustomContextMenu)
         self.customContextMenuRequested.connect(
             lambda p: self._menu.popup(self.mapToGlobal(p + QtCore.QPoint(0, 3)))
         )
@@ -1061,11 +1047,7 @@ class PythonShell(BaseShell):
         self.stateChanged.emit(self)
 
     def connectToKernel(self, info):
-        """connectToKernel()
-
-        Create kernel and connect to it.
-
-        """
+        """Create kernel and connect to it."""
 
         # Create yoton context
         self._context = ct = yoton.Context()
@@ -1107,7 +1089,7 @@ class PythonShell(BaseShell):
         # Connect! The broker will only start the kernel AFTER
         # we connect, so we do not miss out on anything.
         slot = pyzo.localKernelManager.createKernel(finishKernelInfo(info))
-        self._brokerConnection = ct.connect("localhost:%i" % slot)
+        self._brokerConnection = ct.connect("localhost:{}".format(slot))
         self._brokerConnection.closed.bind(self._onConnectionClose)
 
         # Force updating of breakpoints
@@ -1142,8 +1124,7 @@ class PythonShell(BaseShell):
                 self.setParser("python2")
             else:
                 self.setParser("python3")
-            version = [str(v) for v in version]
-            self._version = ".".join(version[:2])
+            self._version = "{}.{}".format(*version[:2])
 
         # Set keywords
         L = startup_info.get("keywords", None)
@@ -1174,7 +1155,10 @@ class PythonShell(BaseShell):
         self._currentCTO = cto
 
         # Post request
-        future = self._request.signature(cto.name)
+        if cto.useIntermediateResult:
+            future = self._request.signatureWithIntermediateResult(cto.name)
+        else:
+            future = self._request.signature(cto.name)
         future.add_done_callback(self._processCallTip_response)
         future.cto = cto
 
@@ -1201,10 +1185,13 @@ class PythonShell(BaseShell):
             return
 
         # Invalid response
-        if response is None:
+        if not response:
             cto.textCtrl.autocompleteCancel()
             return
 
+        if cto.useIntermediateResult:
+            assert response.startswith("__pyzo__calltip")
+            response = cto.name + response[len("__pyzo__calltip"):]
         # If still required, show tip, otherwise only store result
         if cto is self._currentCTO:
             cto.finish(response)
@@ -1224,6 +1211,8 @@ class PythonShell(BaseShell):
             aco.addNames(self._builtins)
             if pyzo.config.settings.autoComplete_keywords:
                 aco.addNames(self._keywords)
+        elif aco.name == "[":
+            return
 
         # Set buffer to prevent doing a second request
         # and store aco to see whether the response is still wanted.
@@ -1231,7 +1220,16 @@ class PythonShell(BaseShell):
         self._currentACO = aco
 
         # Post request
-        future = self._request.dir(aco.name)
+        if aco.useIntermediateResult:
+            if aco.name.endswith("["):  # if key auto-completion
+                future = self._request.dir2WithIntermediateResult(aco.name[:-1])
+            else:
+                future = self._request.dirWithIntermediateResult(aco.name)
+        else:
+            if aco.name.endswith("["):  # if attribute or name auto-completion
+                future = self._request.dir2(aco.name[:-1])
+            else:
+                future = self._request.dir(aco.name)
         future.add_done_callback(self._processAutoComp_response)
         future.aco = aco
 
@@ -1240,7 +1238,7 @@ class PythonShell(BaseShell):
 
         # Process future
         if future.cancelled():
-            # print('Introspect cancelled') # No living kernel
+            # print('Introspect cancelled')  # No living kernel
             return
         elif future.exception():
             print("Introspect-exception: ", future.exception())
@@ -1260,7 +1258,13 @@ class PythonShell(BaseShell):
         # Add result to the list
         foundNames = []
         if response is not None:
-            foundNames = response
+            if aco.name.endswith("["):  # if key auto-completion
+                foundNames = [name[1:-1] for name, type_, kind, repr_ in response if name[0] == "["]
+                maxKeyEntries = 200
+                if len(foundNames) > maxKeyEntries:
+                    foundNames = foundNames[:maxKeyEntries-1] + ["# ... too many entries"] + foundNames[-1:]
+            else:  # if attribute or name auto-completion
+                foundNames = response
         aco.addNames(foundNames)
 
         # Process list
@@ -1273,12 +1277,12 @@ class PythonShell(BaseShell):
                 line = importLines[baseName].strip()
                 if line not in self._importAttempts:
                     # Do import
-                    self.processLine(line + " # auto-import")
+                    self.processLine(line + "  # auto-import")
                     self._importAttempts.append(line)
                     # Wait a barely noticable time to increase the chances
-                    # That the import is complete when we repost the request.
+                    # that the import is complete when we repost the request.
                     time.sleep(0.2)
-                    # To be sure, decrease the experiration date on the buffer
+                    # To be sure, decrease the expiration date on the buffer
                     aco.setBuffer(timeout=1)
                     # Repost request
                     future = self._request.signature(aco.name)
@@ -1294,9 +1298,7 @@ class PythonShell(BaseShell):
     ## Methods for executing code
 
     def executeCommand(self, text):
-        """executeCommand(text)
-        Execute one-line command in the remote Python session.
-        """
+        """Execute one-line command in the remote Python session."""
 
         # Ensure edit line is selected (to reset scrolling to end)
         self.ensureCursorAtEditLine()
@@ -1304,8 +1306,8 @@ class PythonShell(BaseShell):
         self._ctrl_command.send(text)
 
     def executeCode(self, text, fname, lineno=None, cellName=None, changeDir=False):
-        """executeCode(text, fname, lineno, cellName=None)
-        Execute (run) a large piece of code in the remote shell.
+        """Execute (run) a large piece of code in the remote shell.
+
         text: the source code to execute
         filename: the file from which the source comes
         lineno: the first lineno of the text in the file, where 0 would be
@@ -1336,9 +1338,7 @@ class PythonShell(BaseShell):
         lines = text.splitlines()
         lastLineOfCode = 0
         minIndent = 99
-        for linenr in range(len(lines)):
-            # Get line
-            line = lines[linenr]
+        for linenr, line in enumerate(lines):
             # Check if empty (can be commented, but nothing more)
             tmp = line.split("#", 1)[0]  # get part before first #
             if tmp.count(" ") == len(tmp):
@@ -1386,10 +1386,8 @@ class PythonShell(BaseShell):
     ## The polling methods and terminating methods
 
     def poll(self, channel=None):
-        """poll()
-        To keep the shell up-to-date.
-        Call this periodically.
-        """
+        """To keep the shell up-to-date. Call this periodically."""
+        idle = True
 
         if self._write_buffer:
             # There is still data in the buffer
@@ -1418,6 +1416,7 @@ class PythonShell(BaseShell):
 
         # Write all pending messages that are later than any other message
         if sub:
+            idle = False
             # Select messages to process
             N = 256
             M, buffer = M[:N], M[N:]
@@ -1432,20 +1431,22 @@ class PythonShell(BaseShell):
                 prompt = 1
             elif sub is self._strm_prompt:
                 prompt = 2
+                M = M[-1:]  # only use the newest prompt
             # Get color
             color = None
             if sub is self._strm_broker:
-                color = "#000"
+                color = "#fff" if pyzo.darkSyntax else "#000"
             elif sub is self._strm_raw:
-                color = "#888888"  # Halfway
+                color = "#bbb" if pyzo.darkSyntax else "#888888"
             elif sub is self._strm_err:
-                color = "#F00"
+                color = "#f00"
             # Write
             self.write("".join(M), prompt, color)
 
         # Do any actions?
         action = self._strm_action.recv(False)
         if action:
+            idle = False
             if action == "cls":
                 self.clearScreen()
             elif action.startswith("open "):
@@ -1461,10 +1462,14 @@ class PythonShell(BaseShell):
                 if editor and linenr:
                     editor._editor.gotoLine(linenr)
             else:
-                print("Unkown action: %s" % action)
+                print("Unknown action: {}".format(action))
 
         # ----- status
 
+        newInterval = POLL_TIMER_INTERVAL_IDLE if idle else POLL_TIMER_INTERVAL_BUSY
+        if self._timer.interval() != newInterval:
+            self._timer.setInterval(newInterval)
+
         # Do not update status when the kernel is not really up and running
         # self._version is set when the startup info is received
         if not self._version:
@@ -1483,18 +1488,22 @@ class PythonShell(BaseShell):
             self.debugStateChanged.emit(self)
 
     def interrupt(self):
-        """interrupt()
-        Send a Keyboard interrupt signal to the main thread of the
-        remote process.
-        """
+        """Send a Keyboard interrupt signal to the main thread of the remote process."""
         # Ensure edit line is selected (to reset scrolling to end)
         self.ensureCursorAtEditLine()
 
         self._ctrl_broker.send("INT")
 
+    def pause(self):
+        """Send a pause signal to the main thread of the remote process."""
+        # Ensure edit line is selected (to reset scrolling to end)
+        self.ensureCursorAtEditLine()
+
+        self._ctrl_broker.send("PAUSE")
+
     def restart(self, scriptFile=None):
-        """restart(scriptFile=None)
-        Terminate the shell, after which it is restarted.
+        """Terminate the shell, after which it is restarted.
+
         Args can be a filename, to execute as a script as soon as the
         shell is back up.
         """
@@ -1513,11 +1522,11 @@ class PythonShell(BaseShell):
         self.resetVariables()
 
     def terminate(self):
-        """terminate()
-        Terminates the python process. It will first try gently, but
-        if that does not work, the process shall be killed.
-        To be notified of the termination, connect to the "terminated"
-        signal of the shell.
+        """Terminates the python process.
+
+        It will first try gently, but if that does not work, the process
+        shall be killed. To be notified of the termination, connect to the
+        "terminated" signal of the shell.
         """
         # Ensure edit line is selected (to reset scrolling to end)
         self.ensureCursorAtEditLine()
@@ -1525,10 +1534,9 @@ class PythonShell(BaseShell):
         self._ctrl_broker.send("TERM")
 
     def closeShell(self):  # do not call it close(); that is a reserved method.
-        """closeShell()
+        """This closes the shell.
 
-        Very simple. This closes the shell. If possible, we will first
-        tell the broker to terminate the kernel.
+        If possible, we will first tell the broker to terminate the kernel.
 
         The broker will be cleaned up if there are no clients connected
         and if there is no active kernel. In a multi-user environment,
@@ -1539,13 +1547,14 @@ class PythonShell(BaseShell):
         In both cases however, it is the responsibility of the broker to
         terminate the kernel, and the shell will simply assume that this
         will work :)
-
         """
 
+        pyzo.editors.breakPointsChanged.disconnect(self.sendBreakPoints)
+
         # If we can, try to tell the broker to terminate the kernel
         if self._context and self._context.connection_count:
             self.terminate()
-            self._context.flush()  # Important, make sure the message is send!
+            self._context.flush()  # Important, make sure the message is sent!
             self._context.close()
 
         # Adios
@@ -1553,6 +1562,7 @@ class PythonShell(BaseShell):
 
     def _onConnectionClose(self, c, why):
         """To be called after disconnecting.
+
         In general, the broker will not close the connection, so it can
         be considered an error-state if this function is called.
         """
@@ -1562,8 +1572,8 @@ class PythonShell(BaseShell):
             self._context.close()
 
         # New (empty prompt)
-        self._cursor1.movePosition(self._cursor1.End, A_MOVE)
-        self._cursor2.movePosition(self._cursor2.End, A_MOVE)
+        self._cursor1.movePosition(self._cursor1.MoveOperation.End, A_MOVE)
+        self._cursor2.movePosition(self._cursor2.MoveOperation.End, A_MOVE)
 
         self.write("\n\n")
         self.write("Lost connection with broker:\n")
@@ -1575,7 +1585,7 @@ class PythonShell(BaseShell):
 
         # Goto end such that the closing message is visible
         cursor = self.textCursor()
-        cursor.movePosition(cursor.End, A_MOVE)
+        cursor.movePosition(cursor.MoveOperation.End, A_MOVE)
         self.setTextCursor(cursor)
         self.ensureCursorVisible()
 
