@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2016, the Pyzo development team
 #
 # Pyzo is distributed under the terms of the 2-Clause BSD License.
@@ -33,13 +32,13 @@ class ShellInfoLineEdit(QtWidgets.QLineEdit):
 
 class ShellInfo_name(ShellInfoLineEdit):
     def __init__(self, *args, **kwargs):
-        ShellInfoLineEdit.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.editingFinished.connect(self.onValueChanged)
         t = translate("shell", "name ::: The name of this configuration.")
         self.setPlaceholderText(t.tt)
 
     def setTheText(self, value):
-        ShellInfoLineEdit.setTheText(self, value)
+        super().setTheText(value)
         self.onValueChanged()
 
     def onValueChanged(self):
@@ -48,19 +47,19 @@ class ShellInfo_name(ShellInfoLineEdit):
 
 class ShellInfo_exe(QtWidgets.QComboBox):
     def __init__(self, *args):
-        QtWidgets.QComboBox.__init__(self, *args)
+        super().__init__(*args)
 
     def _interpreterName(self, p):
         if p.is_conda:
-            return "%s  [v%s, conda]" % (p.path, p.version)
+            return "{}  [v{}, conda]".format(p.path, p.version)
         else:
-            return "%s  [v%s]" % (p.path, p.version)
+            return "{}  [v{}]".format(p.path, p.version)
 
     def setTheText(self, value):
         # Init
         self.clear()
         self.setEditable(True)
-        self.setInsertPolicy(self.InsertAtTop)
+        self.setInsertPolicy(self.InsertPolicy.InsertAtTop)
 
         # Get known interpreters from shellDialog (which are sorted by version)
         shellDialog = self
@@ -92,26 +91,16 @@ class ShellInfo_exe(QtWidgets.QComboBox):
 
 class ShellInfo_ipython(QtWidgets.QCheckBox):
     def __init__(self, parent):
-        QtWidgets.QCheckBox.__init__(self, parent)
+        super().__init__(parent)
         t = translate("shell", "ipython ::: Use IPython shell if available.")
         self.setText(t.tt)
         self.setChecked(False)
 
     def setTheText(self, value):
-        if value.lower() in [
-            "",
-            "no",
-            "false",
-        ]:  # Also for empty string; default is False
-            self.setChecked(False)
-        else:
-            self.setChecked(True)
+        self.setChecked(value.lower() not in ("", "no", "false"))
 
     def getTheText(self):
-        if self.isChecked():
-            return "yes"
-        else:
-            return "no"
+        return "yes" if self.isChecked() else "no"
 
 
 class ShellInfo_gui(QtWidgets.QComboBox):
@@ -146,11 +135,10 @@ class ShellInfo_gui(QtWidgets.QComboBox):
         # Set options
         ii = 0
         self.clear()
-        for i in range(len(self.GUIS)):
-            gui, des = self.GUIS[i]
+        for i, (gui, des) in enumerate(self.GUIS):
             if value == gui.upper():
                 ii = i
-            self.addItem("%s  -  %s" % (gui, des))
+            self.addItem("{}  -  {}".format(gui, des))
 
         # Set current text
         self.setCurrentIndex(ii)
@@ -166,7 +154,7 @@ class ShellinfoWithSystemDefault(QtWidgets.QVBoxLayout):
 
     def __init__(self, parent, widget):
         # Do not pass parent, because is a sublayout
-        QtWidgets.QVBoxLayout.__init__(self)
+        super().__init__()
 
         # Layout
         self.setSpacing(1)
@@ -236,7 +224,7 @@ class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
         self._edit.textChanged.connect(self.onEditChanged)
 
         # Instantiate
-        ShellinfoWithSystemDefault.__init__(self, parent, self._edit)
+        super().__init__(parent, self._edit)
 
     def getWidgetText(self):
         return self._edit.toPlainText()
@@ -245,7 +233,7 @@ class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
         if value is None:
             pp = os.environ.get("PYTHONPATH", "")
             pp = pp.replace(os.pathsep, "\n").strip()
-            value = "$PYTHONPATH:\n%s\n" % pp
+            value = "$PYTHONPATH:\n{}\n".format(pp)
         self._edit.setText(value)
 
 
@@ -260,7 +248,7 @@ class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
 #         self._edit.textEdited.connect(self.onEditChanged)
 #
 #         # Instantiate
-#         ShellinfoWithSystemDefault.__init__(self, parent, self._edit)
+#         super().__init__(parent, self._edit)
 #
 #
 #     def getWidgetText(self):
@@ -271,7 +259,7 @@ class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
 #         if value is None:
 #             pp = os.environ.get('PYTHONSTARTUP','').strip()
 #             if pp:
-#                 value = '$PYTHONSTARTUP: "%s"' % pp
+#                 value = '$PYTHONSTARTUP: "{}"'.format(pp)
 #             else:
 #                 value = '$PYTHONSTARTUP: None'
 #
@@ -281,17 +269,21 @@ class ShellInfo_pythonPath(ShellinfoWithSystemDefault):
 class ShellInfo_startupScript(QtWidgets.QVBoxLayout):
     DISABLE_SYSTEM_DEFAULT = sys.platform == "darwin"
     SYSTEM_VALUE = "$PYTHONSTARTUP"
-    RUN_AFTER_GUI_TEXT = "# AFTER_GUI - code below runs after integrating the GUI\n"
+    RUN_BEFORE_AFTER_GUI_TEXT = (
+        "... Python code to run before integrating the GUI ...\n"
+        "# AFTER_GUI\n"
+        "... Python code to run after integrating the GUI ..."
+    )
 
     def __init__(self, parent):
         # Do not pass parent, because is a sublayout
-        QtWidgets.QVBoxLayout.__init__(self)
+        super().__init__()
 
         # Create sub-widget
         self._edit1 = QtWidgets.QLineEdit(parent)
         self._edit1.textEdited.connect(self.onEditChanged)
         if sys.platform.startswith("win"):
-            self._edit1.setPlaceholderText("C:\\path\\to\\script.py")
+            self._edit1.setPlaceholderText(r"C:\path\to\script.py")
         else:
             self._edit1.setPlaceholderText("/path/to/script.py")
         #
@@ -300,6 +292,7 @@ class ShellInfo_startupScript(QtWidgets.QVBoxLayout):
         self._edit2.setMaximumHeight(80)
         self._edit2.setMinimumWidth(200)
         self._edit2.textChanged.connect(self.onEditChanged)
+        self._edit2.setPlaceholderText(self.RUN_BEFORE_AFTER_GUI_TEXT)
 
         # Layout
         self.setSpacing(1)
@@ -362,7 +355,7 @@ class ShellInfo_startupScript(QtWidgets.QVBoxLayout):
                 self._radio_system.setChecked(True)
             pp = os.environ.get("PYTHONSTARTUP", "").strip()
             if pp:
-                value = '$PYTHONSTARTUP: "%s"' % pp
+                value = '$PYTHONSTARTUP: "{}"'.format(pp)
             else:
                 value = "$PYTHONSTARTUP: None"
             #
@@ -387,7 +380,7 @@ class ShellInfo_startupScript(QtWidgets.QVBoxLayout):
             self._edit1.hide()
             self._edit2.show()
             if not value.strip():
-                value = self.RUN_AFTER_GUI_TEXT
+                value = ""  # this will display the placeholder text
             self._edit2.setText(value)
 
     def getTheText(self):
@@ -396,16 +389,16 @@ class ShellInfo_startupScript(QtWidgets.QVBoxLayout):
 
 class ShellInfo_startDir(ShellInfoLineEdit):
     def __init__(self, parent):
-        ShellInfoLineEdit.__init__(self, parent)
+        super().__init__(parent)
         if sys.platform.startswith("win"):
-            self.setPlaceholderText("C:\\path\\to\\your\\python\\modules")
+            self.setPlaceholderText(r"C:\path\to\your\python\modules")
         else:
             self.setPlaceholderText("/path/to/your/python/modules")
 
 
 class ShellInfo_argv(ShellInfoLineEdit):
     def __init__(self, parent):
-        ShellInfoLineEdit.__init__(self, parent)
+        super().__init__(parent)
         self.setPlaceholderText('arg1 arg2 "arg with spaces"')
 
 
@@ -413,7 +406,7 @@ class ShellInfo_environ(QtWidgets.QTextEdit):
     EXAMPLE = "PYZO_PROCESS_EVENTS_WHILE_DEBUGGING=1\nEXAMPLE_VAR1=value1"
 
     def __init__(self, parent):
-        QtWidgets.QTextEdit.__init__(self, parent)
+        super().__init__(parent)
         self.zoomOut(1)
         self.setPlaceholderText(self.EXAMPLE)
 
@@ -456,13 +449,13 @@ class ShellInfoTab(QtWidgets.QScrollArea):
     ]
 
     def __init__(self, parent):
-        QtWidgets.QScrollArea.__init__(self, parent)
+        super().__init__(parent)
 
         # Init the scroll area
-        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
-        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
+        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarPolicy.ScrollBarAsNeeded)
         self.setWidgetResizable(True)
-        self.setFrameShape(QtWidgets.QFrame.NoFrame)
+        self.setFrameShape(QtWidgets.QFrame.Shape.NoFrame)
 
         # Create widget and a layout
         self._content = QtWidgets.QWidget(parent)
@@ -487,19 +480,28 @@ class ShellInfoTab(QtWidgets.QScrollArea):
             # Add to layout
             self._formLayout.addRow(label, instance)
 
-        # Add delete button
+        buttonsLayout = QtWidgets.QHBoxLayout()
 
+        # Add delete button
         t = translate("shell", "Delete ::: Delete this shell configuration")
-        label = QtWidgets.QLabel("", self._content)
         instance = QtWidgets.QPushButton(pyzo.icons.cancel, t, self._content)
         instance.setToolTip(t.tt)
         instance.setAutoDefault(False)
         instance.clicked.connect(self.parent().parent().onTabClose)
-        deleteLayout = QtWidgets.QHBoxLayout()
-        deleteLayout.addWidget(instance, 0)
-        deleteLayout.addStretch(1)
+        buttonsLayout.addWidget(instance, 0)
+
+        # Add copy button
+        t = translate("shell", "Copy ::: Copy this shell configuration")
+        instance = QtWidgets.QPushButton(pyzo.icons.page_white_copy, t, self._content)
+        instance.setToolTip(t.tt)
+        instance.setAutoDefault(False)
+        instance.clicked.connect(self.parent().parent().onTabCopy)
+        buttonsLayout.addWidget(instance, 0)
+
+        buttonsLayout.addStretch(1)
         # Add to layout
-        self._formLayout.addRow(label, deleteLayout)
+        label = QtWidgets.QLabel("", self._content)
+        self._formLayout.addRow(label, buttonsLayout)
 
         # Apply layout
         self._formLayout.setSpacing(15)
@@ -521,7 +523,7 @@ class ShellInfoTab(QtWidgets.QScrollArea):
             # Name
             n = self.parent().parent().count()
             if n > 1:
-                info.name = "Shell config %i" % n
+                info.name = "Shell config {}".format(n)
 
         # Store info
         self._info = info
@@ -557,14 +559,14 @@ class ShellInfoDialog(QtWidgets.QDialog):
     """Dialog to edit the shell configurations."""
 
     def __init__(self, *args):
-        QtWidgets.QDialog.__init__(self, *args)
+        super().__init__(*args)
         self.setModal(True)
 
         # Set title
         self.setWindowTitle(pyzo.translate("shell", "Shell configurations"))
         # Create tab widget
         self._tabs = QtWidgets.QTabWidget(self)
-        # self._tabs = CompactTabWidget(self, padding=(4,4,5,5))
+        # self._tabs = CompactTabWidget(self, padding=(4, 4, 5, 5))
         # self._tabs.setDocumentMode(False)
         self._tabs.setMovable(True)
 
@@ -590,7 +592,7 @@ class ShellInfoDialog(QtWidgets.QDialog):
         self._add = QtWidgets.QToolButton(self)
         self._tabs.setCornerWidget(self._add)
         self._add.clicked.connect(self.onAdd)
-        self._add.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
+        self._add.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         self._add.setIcon(pyzo.icons.add)
         self._add.setText(translate("shell", "Add config"))
         #
@@ -635,6 +637,20 @@ class ShellInfoDialog(QtWidgets.QDialog):
         index = self._tabs.currentIndex()
         self._tabs.removeTab(index)
 
+    def onTabCopy(self):
+        # Get original widget
+        w0 = self._tabs.currentWidget()
+        # Build new info
+        info = w0.getInfo().copy()
+        info.name += " (2)"
+        # Create widget and add to tabs
+        w = ShellInfoTab(self._tabs)
+        self._tabs.insertTab(self._tabs.indexOf(w0) + 1, w, "---")
+        w.setInfo(info)
+        # Select
+        self._tabs.setCurrentWidget(w)
+        w.setFocus()
+
     def applyAndClose(self, event=None):
         self.apply()
         self.close()
