@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2016, the Pyzo development team
 #
 # Pyzo is distributed under the terms of the 2-Clause BSD License.
@@ -302,7 +301,7 @@ htmlWrap = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN"
 <html>
 <head>
 <style type="text/css">
-pre, code {{background-color: #F2F2F2;}}
+pre, code {{background-color: BGCOLOR_CODE;}}
 </style>
 </head>
 <body style=" font-family:'Sans Serif'; font-size:{}pt; font-weight:400; font-style:normal;">
@@ -314,7 +313,8 @@ pre, code {{background-color: #F2F2F2;}}
 
 # Define title text (font-size percentage does not seem to work sadly.)
 def get_title_text(objectName, h_class="", h_repr=""):
-    title_text = "<p style='background-color:#def;'>"
+    title_bgcolor = "#468" if pyzo.darkQt else "#def"
+    title_text = "<p style='background-color:{};'>".format(title_bgcolor)
     if h_class == "~python_keyword~":
         title_text += "<b>Keyword:</b> {}".format(objectName)
     elif h_class == "~python_operator~":
@@ -338,8 +338,9 @@ initText = pyzo.translate(
     "pyzoInteractiveHelp",
     """
 Help information is queried from the current shell
-when moving up/down in the autocompletion list
-and when double clicking on a name.
+automatically when moving up/down in the autocompletion list,
+or manually via the context menu in the shell and editor,
+or via the Workspace tool.
 """,
 )
 
@@ -367,7 +368,7 @@ class PyzoInteractiveHelpHistoryMenu(QtWidgets.QMenu):
 
 class PyzoInteractiveHelp(QtWidgets.QWidget):
     def __init__(self, parent):
-        QtWidgets.QWidget.__init__(self, parent)
+        super().__init__(parent)
 
         # Create text field, checkbox, and button
         self._text = QtWidgets.QLineEdit(self)
@@ -376,27 +377,33 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
         style = QtWidgets.qApp.style()
 
         self._backBut = QtWidgets.QToolButton(self)
-        self._backBut.setIcon(style.standardIcon(style.SP_ArrowLeft))
+        self._backBut.setIcon(style.standardIcon(style.StandardPixmap.SP_ArrowLeft))
         self._backBut.setIconSize(QtCore.QSize(16, 16))
-        self._backBut.setPopupMode(self._backBut.DelayedPopup)
+        self._backBut.setPopupMode(self._backBut.ToolButtonPopupMode.DelayedPopup)
         self._backBut.setMenu(
             PyzoInteractiveHelpHistoryMenu("Backward menu", self, False)
         )
 
         self._forwBut = QtWidgets.QToolButton(self)
-        self._forwBut.setIcon(style.standardIcon(style.SP_ArrowRight))
+        self._forwBut.setIcon(style.standardIcon(style.StandardPixmap.SP_ArrowRight))
         self._forwBut.setIconSize(QtCore.QSize(16, 16))
-        self._forwBut.setPopupMode(self._forwBut.DelayedPopup)
+        self._forwBut.setPopupMode(self._forwBut.ToolButtonPopupMode.DelayedPopup)
         self._forwBut.setMenu(
             PyzoInteractiveHelpHistoryMenu("Forward menu", self, True)
         )
 
+        self._pauseBut = QtWidgets.QToolButton(self)
+        self._pauseBut.setIcon(style.standardIcon(style.StandardPixmap.SP_MediaPause))
+        self._pauseBut.setIconSize(QtCore.QSize(16, 16))
+        self._pauseBut.setCheckable(True)
+        self._pauseBut.setToolTip("Pause automatic help from autocompletion")
+
         # Create options button
         self._options = QtWidgets.QToolButton(self)
         self._options.setIcon(pyzo.icons.wrench)
         self._options.setIconSize(QtCore.QSize(16, 16))
-        self._options.setPopupMode(self._options.InstantPopup)
-        self._options.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
+        self._options.setPopupMode(self._options.ToolButtonPopupMode.InstantPopup)
+        self._options.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
 
         # Create options menu
         self._options._menu = QtWidgets.QMenu()
@@ -405,7 +412,7 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
         # Create browser
         self._browser = QtWidgets.QTextBrowser(self)
         self._browser_text = initText
-        self._browser.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+        self._browser.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.CustomContextMenu)
         self._browser.customContextMenuRequested.connect(self.showMenu)
 
         # Create two sizers
@@ -415,6 +422,7 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
         # Put the elements together
         self._sizer2.addWidget(self._backBut, 1)
         self._sizer2.addWidget(self._forwBut, 2)
+        self._sizer2.addWidget(self._pauseBut, 1)
         self._sizer2.addWidget(self._text, 4)
         self._sizer2.addWidget(self._printBut, 0)
         self._sizer2.addWidget(self._options, 3)
@@ -446,6 +454,7 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
         self._printBut.clicked.connect(self.printDoc)
         self._backBut.clicked.connect(self.goBack)
         self._forwBut.clicked.connect(self.goForward)
+        self._pauseBut.clicked.connect(self.onTogglePause)
         #
         self._options.pressed.connect(self.onOptionsPress)
         self._options._menu.triggered.connect(self.onOptionMenuTiggered)
@@ -458,25 +467,26 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
 
     def showMenu(self, pos):
         menu = self._browser.createStandardContextMenu()
-        help = QtWidgets.QAction(
+        help = QtGui.QAction(
             pyzo.icons.help, pyzo.translate("pyzoInteractiveHelp", "Help on this"), menu
         )
         help.triggered.connect(partial(self.helpOnThis, pos=pos))
         menu.insertAction(menu.actions()[0], help)
-        menu.exec(self.mapToGlobal(pos))
+        menu.exec_(self.mapToGlobal(pos))
 
     def helpOnThis(self, pos):
         name = self._browser.textCursor().selectedText().strip()
         if name == "":
             cursor = self._browser.cursorForPosition(pos)
-            cursor.select(cursor.WordUnderCursor)
+            cursor.select(cursor.SelectionType.WordUnderCursor)
             name = cursor.selectedText()
         if name != "":
             self.setObjectName(name, True)
 
     def onOptionsPress(self):
         """Create the menu for the button, Do each time to make sure
-        the checks are right."""
+        the checks are right.
+        """
 
         # Get menu
         menu = self._options._menu
@@ -495,9 +505,9 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
         currentSize = self._config.fontSize
         for i in range(8, 15):
             action = menu.addAction(
-                pyzo.translate("pyzoInteractiveHelp", "Font size: %i") % i
+                pyzo.translate("pyzoInteractiveHelp", "Font size: {}").format(i)
             )
-            action._what = "font-size: %ipx" % i
+            action._what = "font-size: {}px".format(i)
             action.setCheckable(True)
             action.setChecked(i == currentSize)
 
@@ -531,15 +541,22 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
 
         # Set text with html header
         size = self._config.fontSize
-        self._browser.setHtml(htmlWrap.format(size, text))
+        self._browser.setHtml(
+            htmlWrap.replace(
+                "BGCOLOR_CODE", "#505050" if pyzo.darkQt else "#f2f2f2"
+            ).format(size, text)
+        )
 
     def setObjectName(self, name, addToHist=False):
         """Set the object name programatically
-        and query documentation for it."""
+        and query documentation for it.
+        """
         self._text.setText(name)
         self.queryDoc(addToHist)
 
     def helpFromCompletion(self, name, addToHist=False):
+        if self._pauseBut.isChecked():
+            return
         self.setObjectName(name, addToHist)
 
     def currentHist(self):
@@ -549,13 +566,15 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
             return None
 
     def addToHist(self, name):
-        if name == self.currentHist():
+        if not name or name == self.currentHist():
             return
         self._history = self._history[: self._histindex + 1]
         self._history.append(name)
         self._histindex = len(self._history) - 1
 
     def restoreCurrent(self):
+        if self._pauseBut.isChecked():
+            return
         self.setObjectName(self.currentHist())
 
     def goBack(self):
@@ -568,6 +587,10 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
             self._histindex += 1
             self.setObjectName(self.currentHist())
 
+    def onTogglePause(self, pauseActivated):
+        if pauseActivated:
+            self.addToHist(self._text.text())
+
     def printDoc(self):
         """Print the doc for the text in the line edit."""
         # Get name
@@ -659,13 +682,15 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
                     name = objectName.split(".")[-1]
                     # Is the signature in the docstring?
                     docs = h_text.replace("\n", "|")
-                    tmp = re.search("[a-zA-z_\.]*?" + name + "\(.*?\)", docs)
+                    tmp = re.search(
+                        r"[a-zA-Z0-9_\.]*?" + re.escape(name) + r"\(.*?\)", docs
+                    )
                     if tmp and tmp.span(0)[0] < 5:
                         header = tmp.group(0)
                         h_text = h_text[len(header) :].lstrip(":").lstrip()
                         header = header.replace("|", "")
                         # h_text = header + '\n\n' + h_text
-                    elif h_text.startswith(objectName) or h_text.startswith(name):
+                    elif h_text.startswith((objectName, name)):
                         header, sep, docs = h_text.partition("\n")
                         # h_text = header + '\n\n' + docs
                         h_text = docs
@@ -674,13 +699,14 @@ class PyzoInteractiveHelp(QtWidgets.QWidget):
 
                 # Parse the text as rest/numpy like docstring
                 h_text = self.smartFormat(h_text)
-                h_text = re.sub("``(.*?)``", r"<code>\1</code>", h_text)
+                h_text = re.sub(r"``(.*?)``", r"<code>\1</code>", h_text)
                 if header:
-                    h_text = "<p style='color:#005;'><b>%s</b></p>\n%s" % (
+                    h_text = "<p style='color:{};'><b>{}</b></p>\n{}".format(
+                        "#0bf" if pyzo.darkQt else "#005",
                         header,
                         h_text,
                     )
-                    # h_text = "<b>%s</b><br /><br />\n%s" % (header, h_text)
+                    # h_text = "<b>{}</b><br /><br />\n{}".format(header, h_text)
             else:
                 # Make newlines html
                 h_text = h_text.replace("\n", "<br />")
